@@ -42,16 +42,44 @@
 #include <p4est_bits.h>
 #include <p4est_extended.h>
 #include <p4est_iterate.h>
+#include <p4est_io.h>
+#include <p4est_communication.h>
 #else
 #include <p8est_vtk.h>
 #include <p8est_bits.h>
 #include <p8est_extended.h>
 #include <p8est_iterate.h>
+#include <p8est_io.h>
+#include <p8est_communication.h>
 #endif
+#include <sc_options.h>
+
+/* The file format and API for checkpoint load/save will change */
+#ifdef P4EST_ENABLE_FILE_DEPRECATED
+
+#ifndef P4_TO_P8
+#define P4EST_DATA_FILE_EXT "p4d" /**< file extension of p4est data files */
+#else
+#define P4EST_DATA_FILE_EXT               P8EST_DATA_FILE_EXT
+#define P8EST_DATA_FILE_EXT "p8d" /**< file extension of p8est data files */
+#endif
+
+#define STEP3_BLOCK_SIZE (sizeof (step3_ctx_t)) /**< number of bytes of
+                                                      the simulation context */
+#define STEP3_ENDIAN_CHECK 0x30415062   /* check endian; bPA0 in ASCII */
+
+#endif /* P4EST_ENABLE_FILE_DEPRECATED */
 
 /** We had 1. / 0. here to create a NaN but that is not portable. */
 static const double step3_invalid = -1.;
 
+#ifdef P4EST_ENABLE_FILE_DEPRECATED
+
+/** A Boolean to decide if checkpoint files are written to disk. */
+static int          step3_checkpoint = 0;
+
+#endif /* P4EST_ENABLE_FILE_DEPRECATED */
+
 /* In this example we store data with each quadrant/octant. */
 
 /** Per-quadrant data for this example.
@@ -88,6 +116,9 @@ typedef struct step3_ctx
                                                between repartitioning */
   int                 write_period;       /**< the number of time steps
                                                between writing vtk files */
+  double              current_time;       /**< the current time */
+  int                 time_step;          /**< current time step
+                                               counted from the first start. */
 }
 step3_ctx_t;
 
@@ -518,11 +549,13 @@ step3_interpolate_solution (p4est_iter_volume_info_t * info, void *user_data)
 static void
 step3_write_solution (p4est_t * p4est, int timestep)
 {
-  char                filename[BUFSIZ] = { '\0' };
+  char                filename[BUFSIZ] = "";
+  int                 retval;
   sc_array_t         *u_interp;
   p4est_locidx_t      numquads;
+  p4est_vtk_context_t *context;
 
-  snprintf (filename, 17, P4EST_STRING "_step3_%04d", timestep);
+  snprintf (filename, BUFSIZ, P4EST_STRING "_step3_%04d", timestep);
 
   numquads = p4est->local_num_quadrants;
 
@@ -545,7 +578,7 @@ step3_write_solution (p4est_t * p4est, int timestep)
                  NULL);         /* there is no callback for the corners between quadrants */
 
   /* create VTK output context and set its parameters */
-  p4est_vtk_context_t *context = p4est_vtk_context_new (p4est, filename);
+  context = p4est_vtk_context_new (p4est, filename);
   p4est_vtk_context_set_scale (context, 0.99);  /* quadrant at almost full scale */
 
   /* begin writing the output files */
@@ -570,12 +603,246 @@ step3_write_solution (p4est_t * p4est, int timestep)
   SC_CHECK_ABORT (context != NULL,
                   P4EST_STRING "_vtk: Error writing cell data");
 
-  const int           retval = p4est_vtk_write_footer (context);
+  retval = p4est_vtk_write_footer (context);
   SC_CHECK_ABORT (!retval, P4EST_STRING "_vtk: Error writing footer");
 
   sc_array_destroy (u_interp);
 }
 
+#ifdef P4EST_ENABLE_FILE_DEPRECATED
+
+/** Write a checkpoint file of the current simulation.
+ * The file can be loaded using \ref step3_restart to
+ * restart the simulation.
+ * The checkpoint file is not compiler independent since structures
+ * may be padded by the compiler (e. g., sizeof (step3_data_t)).
+ * This can result in different section sizes.
+ *
+ * \param [in] p4est    the forest, whose quadrant data contains the state
+ * \param [in] timestep the timestep number, used to name the output files
+ */
+static void
+step3_write_checkpoint (p4est_t * p4est, int timestep)
+{
+  char                filename[BUFSIZ] = "";
+  char                user_string[P4EST_FILE_USER_STRING_BYTES] = "";
+  char                quad_data_user_string[P4EST_FILE_USER_STRING_BYTES] =
+    "";
+  int                 errcode;
+  p4est_file_context_t *fc;
+  uint32_t            check_endianness = STEP3_ENDIAN_CHECK;
+  sc_array_t          block_arr;
+
+  /** To write the data to the checkpoint file we need to store it
+   * in a linear array. Therefore, we first need to create such a linear
+   * array consisting of the quadrant data that is not stored in a linear
+   * array in this example code. In this case one could also use
+   * p4est_{load,save} to read and write the p4est including its
+   * quadrant data but we use the p4est_file functions for
+   * demonstration purposes. The p4est_file functions allow us to
+   * store less data than p4est_{load,save} and the p4est_file
+   * functions are required if an application uses external data
+   * storage, that is there is data that is not stored as quadrant
+   * data but for example in a linear array associated by its indexing
+   * to quadrants of a p4est.
+   */
+
+  snprintf (filename, BUFSIZ,
+            P4EST_STRING "_step3_checkpoint%04d." P4EST_DATA_FILE_EXT,
+            timestep);
+
+  fc = p4est_file_open_create (p4est, filename, "Checkpoint file", &errcode);
+  /* One could use the error codes in \ref p4est_io.h and \ref p4est_file_error_string
+   * for more sophisticated error handling.
+   */
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_open_create: Error creating file");
+
+  snprintf (user_string, P4EST_FILE_USER_STRING_BYTES, "%s", "Endianness");
+
+  sc_array_init_data (&block_arr, &check_endianness, sizeof (uint32_t), 1);
+  /* write data to check endianness */
+  fc =
+    p4est_file_write_block (fc, sizeof (uint32_t), &block_arr,
+                            user_string, &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_write_block: Error writing endianness");
+
+  snprintf (user_string, P4EST_FILE_USER_STRING_BYTES, "%s",
+            "Simulation context");
+
+  sc_array_init_data (&block_arr, p4est->user_pointer, STEP3_BLOCK_SIZE, 1);
+  /* write the simulation context */
+  fc =
+    p4est_file_write_block (fc, STEP3_BLOCK_SIZE, &block_arr,
+                            user_string, &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING
+                  "_file_write_block: Error writing simulation context");
+
+  snprintf (user_string, P4EST_FILE_USER_STRING_BYTES,
+            P4EST_STRING " connecitivity");
+
+  /* write connectivity of the p4est */
+  fc =
+    p4est_file_write_connectivity (fc, p4est->connectivity, user_string,
+                                   &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING
+                  "_file_write_connectivity: Error writing connectivity");
+
+  snprintf (user_string, P4EST_FILE_USER_STRING_BYTES,
+            "Quadrants of time step %04d.", timestep);
+  snprintf (quad_data_user_string, P4EST_FILE_USER_STRING_BYTES,
+            "Quadrant data of time step %04d.", timestep);
+
+  /** Write the current p4est to the checkpoint file; we do not write the
+   * connectivity to disk because the connectivity is always the same in
+   * this example and can be created again for each restart.
+   */
+  fc = p4est_file_write_p4est (fc, p4est,
+                               user_string, quad_data_user_string, &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_write_field: Error writing p4est");
+
+  p4est_file_close (fc, &errcode);
+  SC_CHECK_ABORT (errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_close: Error closing file");
+}
+
+#endif /* P4EST_ENABLE_FILE_DEPRECATED */
+
+static void         step3_timestep (p4est_t * p4est, double start_time,
+                                    double end_time);
+
+static void         step3_run (sc_MPI_Comm mpicomm);
+
+/** Load a checkpoint file to restart the simulation.
+ * The checkpoint file is not compiler independent since structures
+ * may be padded by the compiler (e. g., sizeof (step3_data_t)).
+ * This can result in different section sizes.
+ *
+ * \param [in] filename The file path to the checkpoint file
+ *                      created using \ref step3_write_checkpoint.
+ * \param [in] mpicomm  The MPI communciatior that is used for
+ *                      the parallel simulation.
+ * \param [in] time_inc The time increment added to the current time
+ *                      of the restarted simulation to obtain the new
+ *                      end time.
+ */
+static void
+step3_restart (const char *filename, sc_MPI_Comm mpicomm, double time_inc)
+{
+#ifdef P4EST_ENABLE_FILE_DEPRECATED
+  int                 errcode;
+  char                user_string[P4EST_FILE_USER_STRING_BYTES],
+    quad_string[P4EST_FILE_USER_STRING_BYTES],
+    quad_data_string[P4EST_FILE_USER_STRING_BYTES];
+  step3_ctx_t         ctx;
+  p4est_gloidx_t      global_num_quadrants;
+  p4est_file_context_t *fc;
+  p4est_t            *loaded_p4est;
+  p4est_connectivity_t *conn;
+  uint32_t            check_endianness = STEP3_ENDIAN_CHECK;
+  int32_t             read_check_endianness;
+  sc_array_t          block_arr;
+#endif /* P4EST_ENABLE_FILE_DEPRECATED */
+
+  if (filename == NULL) {
+    /* we do not restart */
+    step3_run (mpicomm);
+    return;
+  }
+#ifndef P4EST_ENABLE_FILE_DEPRECATED
+  else {
+    /** If the deprecated file format is not activated, the filename
+     * must be not set.
+     */
+    SC_ABORT_NOT_REACHED ();
+  }
+#else
+  /* we use file I/O to load a checkpoint.  API will change */
+  fc =
+    p4est_file_open_read_ext (mpicomm, filename, user_string,
+                              &global_num_quadrants, &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_open_read: Error opening file");
+
+  sc_array_init_data (&block_arr, &read_check_endianness, sizeof (uint32_t),
+                      1);
+  /* read data endianness */
+  fc =
+    p4est_file_read_block (fc, sizeof (uint32_t), &block_arr,
+                           user_string, &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING
+                  "_file_read_block: Error reading data endianness");
+  P4EST_GLOBAL_PRODUCTIONF ("Read data with user string: %s\n", user_string);
+
+  /* Instead of handling the wrong endianness, we just abort on it.
+   * In a more sophisticated application the data should be converted.
+   */
+  SC_CHECK_ABORT (memcmp
+                  (&read_check_endianness, &check_endianness,
+                   sizeof (uint32_t)) == 0, "Wrong endianness");
+
+  sc_array_init_data (&block_arr, &ctx, STEP3_BLOCK_SIZE, 1);
+  /* read the simulation context */
+  fc =
+    p4est_file_read_block (fc, STEP3_BLOCK_SIZE, &block_arr, user_string,
+                           &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING
+                  "_file_read_block: Error reading simulation context");
+  P4EST_GLOBAL_PRODUCTIONF ("Read data with user string: %s\n", user_string);
+
+  /* read the connectivity */
+  fc = p4est_file_read_connectivity (fc, &conn, user_string, &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING
+                  "_file_read_connectivity: Error reading connectivity");
+  P4EST_ASSERT (conn != NULL);
+  P4EST_GLOBAL_PRODUCTIONF ("Read data with user string: %s\n", user_string);
+
+  fc = p4est_file_read_p4est (fc, conn,
+                              sizeof (step3_data_t),
+                              &loaded_p4est, quad_string, quad_data_string,
+                              &errcode);
+  SC_CHECK_ABORT (fc != NULL
+                  && errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_read_field_ext: Error reading p4est");
+  P4EST_GLOBAL_PRODUCTIONF ("Read quadrants with user string: %s\n",
+                            quad_string);
+  P4EST_GLOBAL_PRODUCTIONF ("Read quadrant data with user string: %s\n",
+                            quad_data_string);
+
+  /* assign simulation context pointer */
+  loaded_p4est->user_pointer = (void *) block_arr.array;
+
+  /* close the file */
+  p4est_file_close (fc, &errcode);
+  SC_CHECK_ABORT (errcode == P4EST_FILE_ERR_SUCCESS,
+                  P4EST_STRING "_file_close: Error closing file");
+
+  step3_timestep (loaded_p4est, ctx.current_time,
+                  ctx.current_time + time_inc);
+
+  /* clean up */
+  conn = loaded_p4est->connectivity;
+  p4est_destroy (loaded_p4est);
+  p4est_connectivity_destroy (conn);
+#endif /* P4EST_ENABLE_FILE_DEPRECATED */
+}
+
 /** Approximate the divergence of (vu) on each quadrant
  *
  * We use piecewise constant approximations on each quadrant, so the value is
@@ -990,12 +1257,13 @@ step3_get_timestep (p4est_t * p4est)
  * Update the state, refine, repartition, and write the solution to file.
  *
  * \param [in,out] p4est the forest, whose state is updated
- * \param [in] time      the end time
+ * \param [in] start_time    simulation start time
+ * \param [in] end_time      simulation end time
  */
 static void
-step3_timestep (p4est_t * p4est, double time)
+step3_timestep (p4est_t * p4est, double start_time, double end_time)
 {
-  double              t = 0.;
+  double              t = start_time;
   double              dt = 0.;
   int                 i;
   step3_data_t       *ghost_data;
@@ -1028,7 +1296,7 @@ step3_timestep (p4est_t * p4est, double time)
 #endif
                  NULL);         /* there is no callback for the corners between quadrants */
 
-  for (t = 0., i = 0; t < time; t += dt, i++) {
+  for (t = start_time, i = ctx->time_step; t < end_time; t += dt, i++) {
     P4EST_GLOBAL_PRODUCTIONF ("time %f\n", t);
 
     /* refine */
@@ -1132,43 +1400,47 @@ step3_timestep (p4est_t * p4est, double time)
                    NULL,        /* there is no callback for the edges between quadrants */
 #endif
                    NULL);       /* there is no callback for the corners between quadrants */
+
   }
 
+  ctx->time_step = i - 1;
+  ctx->current_time = t - dt;
+
+#ifdef P4EST_ENABLE_FILE_DEPRECATED
+
+  if (step3_checkpoint) {
+    /* write checkpoint file */
+    step3_write_checkpoint (p4est, i - 1);
+  }
+
+#endif /* P4EST_ENABLE_FILE_DEPRECATED */
+
   P4EST_FREE (ghost_data);
   p4est_ghost_destroy (ghost);
 }
 
-/** The main step 3 program.
+/** Run the simulation and store the results as quadrant data.
  *
  * Setup of the example parameters; create the forest, with the state variable
  * stored in the quadrant data; refine, balance, and partition the forest;
  * timestep; clean up, and exit.
- */
-int
-main (int argc, char **argv)
+ *
+ * \param [in] mpicomm  The MPI communicator that is used to
+ *                      create the corresponding simulation
+ *                      p4est.
+*/
+static void
+step3_run (sc_MPI_Comm mpicomm)
 {
-  int                 mpiret;
-  int                 recursive, partforcoarsen;
-  sc_MPI_Comm         mpicomm;
-  p4est_t            *p4est;
   p4est_connectivity_t *conn;
+  p4est_t            *p4est;
+  int                 recursive, partforcoarsen;
   step3_ctx_t         ctx;
 
-  /* Initialize MPI; see sc_mpi.h.
-   * If configure --enable-mpi is given these are true MPI calls.
-   * Else these are dummy functions that simulate a single-processor run. */
-  mpiret = sc_MPI_Init (&argc, &argv);
-  SC_CHECK_MPI (mpiret);
-  mpicomm = sc_MPI_COMM_WORLD;
-
-  /* These functions are optional.  If called they store the MPI rank as a
-   * static variable so subsequent global p4est log messages are only issued
-   * from processor zero.  Here we turn off most of the logging; see sc.h. */
-  sc_init (mpicomm, 1, 1, NULL, SC_LP_ESSENTIAL);
-  p4est_init (NULL, SC_LP_PRODUCTION);
-  P4EST_GLOBAL_PRODUCTIONF
-    ("This is the p4est %dD demo example/steps/%s_step3\n",
-     P4EST_DIM, P4EST_STRING);
+  /* Avoid write of uninitialized bytes (valgrind warning)
+   * due to compiler padding.
+   */
+  memset (&ctx, -1, sizeof (ctx));
 
   ctx.bump_width = 0.1;
   ctx.max_err = 2.e-2;
@@ -1189,6 +1461,7 @@ main (int argc, char **argv)
   ctx.refine_period = 2;
   ctx.repartition_period = 4;
   ctx.write_period = 8;
+  ctx.time_step = 0;
 
   /* Create a forest that consists of just one periodic quadtree/octree. */
 #ifndef P4_TO_P8
@@ -1227,11 +1500,70 @@ main (int argc, char **argv)
   p4est_partition (p4est, partforcoarsen, NULL);
 
   /* time step */
-  step3_timestep (p4est, 0.1);
+  step3_timestep (p4est, 0., 0.1);
 
   /* Destroy the p4est and the connectivity structure. */
   p4est_destroy (p4est);
   p4est_connectivity_destroy (conn);
+}
+
+/** The main step 3 program.
+ */
+int
+main (int argc, char **argv)
+{
+  int                 mpiret, retval;
+  int                 help, usage_error;
+  sc_MPI_Comm         mpicomm;
+  sc_options_t       *opt;
+  const char         *filename;
+
+  /* Initialize MPI; see sc_mpi.h.
+   * If configure --enable-mpi is given these are true MPI calls.
+   * Else these are dummy functions that simulate a single-processor run. */
+  mpiret = sc_MPI_Init (&argc, &argv);
+  SC_CHECK_MPI (mpiret);
+  mpicomm = sc_MPI_COMM_WORLD;
+
+  /* These functions are optional.  If called they store the MPI rank as a
+   * static variable so subsequent global p4est log messages are only issued
+   * from processor zero.  Here we turn off most of the logging; see sc.h. */
+  sc_init (mpicomm, 1, 1, NULL, SC_LP_ESSENTIAL);
+  p4est_init (NULL, SC_LP_PRODUCTION);
+  P4EST_GLOBAL_PRODUCTIONF
+    ("This is the p4est %dD demo example/steps/%s_step3\n",
+     P4EST_DIM, P4EST_STRING);
+
+  /* Read command line options */
+  filename = NULL;
+  opt = sc_options_new (argv[0]);
+  sc_options_add_bool (opt, 'H', "help", &help, 0,
+                       "Print the help string and end the program");
+#ifdef P4EST_ENABLE_FILE_DEPRECATED
+  sc_options_add_bool (opt, 'C', "write-checkpoint", &step3_checkpoint, 0,
+                       "Write checkpoint files to disk");
+  sc_options_add_string (opt, 'L', "load-checkpoint", &filename,
+                         NULL, "Load and start from a checkpoint file");
+#endif
+  retval = sc_options_parse (p4est_package_id, SC_LP_ERROR, opt, argc, argv);
+  usage_error = (retval == -1 || retval < argc);
+
+  /* Action of the main program: error out or run demonstration */
+  if (usage_error || help) {
+    sc_options_print_usage (p4est_package_id, SC_LP_PRODUCTION, opt, NULL);
+  }
+  else {
+    sc_options_print_summary (p4est_package_id, SC_LP_PRODUCTION, opt);
+
+    /** Load a checkpoint file and restart the simulation for
+     * read filename != NULL. For filename == NULL the simulation
+     * is run from the beginning (t = 0).
+     */
+    step3_restart (filename, mpicomm, 0.1);
+  }
+
+  /* End of program -- free allocated memory */
+  sc_options_destroy (opt);
 
   /* Verify that allocations internal to p4est and sc do not leak memory.
    * This should be called if sc_init () has been called earlier. */
@@ -1240,5 +1572,5 @@ main (int argc, char **argv)
   /* This is standard MPI programs.  Without --enable-mpi, this is a dummy. */
   mpiret = sc_MPI_Finalize ();
   SC_CHECK_MPI (mpiret);
-  return 0;
+  return usage_error ? EXIT_FAILURE : EXIT_SUCCESS;
 }
