@@ -47,9 +47,15 @@
 SC_EXTERN_C_BEGIN;
 
 /** The finest level of the quadtree for representing nodes */
+#define P4EST_OLD_MAXLEVEL 30   /* in 2D, the maxlevel has always been 30 */
+
+/** The finest level for representing quadrant midpoint coordinates */
 #define P4EST_MAXLEVEL 30
 
 /** The finest level of the quadtree for representing quadrants */
+#define P4EST_OLD_QMAXLEVEL 29  /* in 2D, the qmaxlevel has always been 29 */
+
+/** The finest level of the quadtree for representing quadrant corners */
 #define P4EST_QMAXLEVEL 29
 
 /** The length of a side of the root quadrant */
@@ -58,6 +64,9 @@ SC_EXTERN_C_BEGIN;
 /** The length of a quadrant of level l */
 #define P4EST_QUADRANT_LEN(l) ((p4est_qcoord_t) 1 << (P4EST_MAXLEVEL - (l)))
 
+/** Create a mask of 1-bits from the left and maxlevel-level zero bits. */
+#define P4EST_QUADRANT_MASK(l) (~(P4EST_QUADRANT_LEN (l) - 1))
+
 /** The offset of the highest (farthest from the origin) quadrant at level l
  */
 #define P4EST_LAST_OFFSET(l) (P4EST_ROOT_LEN - P4EST_QUADRANT_LEN (l))
@@ -71,6 +80,15 @@ typedef struct p4est_quadrant
   int8_t              level,    /**< level of refinement */
                       pad8;     /**< padding */
   int16_t             pad16;    /**< padding */
+  /** Union for quadrant data.
+   *
+   * It is important to notice that \ref piggy1 and \ref piggy2 are only used
+   * internally. Hence, they are not part of the API.
+   *
+   * Usually \ref piggy3 is also not part of the API. The only exception holds
+   * for quadrants in the [ghosts](\ref p4est_ghost_t::ghosts) array of
+   * p4est_ghost_t (cf. documentation of [ghosts](\ref p4est_ghost_t::ghosts)).
+   */
   union p4est_quadrant_data
   {
     void               *user_data;      /**< never changed by p4est */
@@ -85,21 +103,22 @@ typedef struct p4est_quadrant
       p4est_topidx_t      which_tree;
       int                 owner_rank;
     }
-    piggy1; /**< of ghost octants, store the tree and owner rank */
+    piggy1; /**< of ghost octants, store the tree and owner rank; not part of
+                 the API */
     struct
     {
       p4est_topidx_t      which_tree;
       p4est_topidx_t      from_tree;
     }
     piggy2; /**< of transformed octants, store the original tree and the
-                 target tree */
+                 target tree; not part of the API */
     struct
     {
       p4est_topidx_t      which_tree;
       p4est_locidx_t      local_num;
     }
     piggy3; /**< of ghost octants, store the tree and index in the owner's
-                 numbering */
+                 numbering; only part of the API in \ref p4est_ghost_t::ghosts */
   }
   p; /**< a union of additional data attached to a quadrant */
 }
@@ -247,7 +266,7 @@ void                p4est_qcoord_to_vertex (p4est_connectivity_t *
                                             p4est_qcoord_t x,
                                             p4est_qcoord_t y, double vxyz[3]);
 
-/** Create a new forest.
+/** Create a new forest with an initial coarse mesh.
  * The new forest consists of equi-partitioned root quadrants.
  * When there are more processors than trees, some processors are empty.
  *
@@ -362,11 +381,22 @@ void                p4est_balance (p4est_t * p4est,
  * The forest will be partitioned between processors such that they
  * have an approximately equal number of quadrants (or sum of weights).
  *
+ * The user data of a quadrant is transferred along within this function.
+ *
+ * On one process, the function noops and does not call the weight callback.
+ * Otherwise, the weight callback is called once per quadrant in order.
+ *
  * \param [in,out] p4est      The forest that will be partitioned.
  * \param [in]     allow_for_coarsening Slightly modify partition such that
  *                            quadrant families are not split between ranks.
  * \param [in]     weight_fn  A weighting function or NULL
  *                            for uniform partitioning.
+ *                            When running with mpisize == 1, never called.
+ *                            Otherwise, called in order for all quadrants
+ *                            if not NULL. A weighting function with constant
+ *                            weight 1 on each quadrant is equivalent
+ *                            to weight_fn == NULL but other constant weightings
+ *                            may result in different uniform partitionings.
  */
 void                p4est_partition (p4est_t * p4est,
                                      int allow_for_coarsening,
@@ -374,10 +404,15 @@ void                p4est_partition (p4est_t * p4est,
 
 /** Compute the checksum for a forest.
  * Based on quadrant arrays only. It is independent of partition and mpisize.
- * \return  Returns the checksum on processor 0 only. 0 on other processors.
+ * \return  Returns the checksum on all processors.
  */
 unsigned            p4est_checksum (p4est_t * p4est);
 
+/** Compute a partition-dependent checksum for a forest.
+ * \return  Returns the checksum on all processors.
+ */
+unsigned            p4est_checksum_partition (p4est_t * p4est);
+
 /** Save the complete connectivity/p4est data to disk.
  *
  * This is a collective operation that all MPI processes need to call.  All
@@ -462,14 +497,44 @@ p4est_quadrant_array_index (sc_array_t * array, size_t it)
   return (p4est_quadrant_t *) (array->array + sizeof (p4est_quadrant_t) * it);
 }
 
-/** Call sc_array_push for a quadrant array. */
+/** Push the copy of a fully initialized quadrant onto a quadrant array.
+ * \param [in,out] array        Valid array of quadrants is pushed to.
+ * \param [in] qsrc     Pointer to a quadrant with fully initialized memory.
+ *                      This means all bits in the quadrant mush have been
+ *                      written to at least once, even the compiler padding.
+ *                      This serves to make the function clean for valgrind.
+ * \return              Newly allocated quadrant with contents of \a qsrc.
+ */
+static inline p4est_quadrant_t *
+p4est_quadrant_array_push_copy (sc_array_t * array,
+                                const p4est_quadrant_t *qsrc)
+{
+  p4est_quadrant_t *q;
+
+  P4EST_ASSERT (array->elem_size == sizeof (p4est_quadrant_t));
+
+  q = (p4est_quadrant_t *) sc_array_push (array);
+  *q = *qsrc;
+  return q;
+}
+
+/** Call sc_array_push for a quadrant array and fully initialize memory.
+ * \param [in,out] array        Valid array of quadrants is pushed to.
+ * \return              Pushed array element with fully initialized memory.
+ *                      In this case, we're writing to all bits of it.
+ *                      This serves to make the quadrant clean for valgrind.
+ */
 /*@unused@*/
 static inline p4est_quadrant_t *
 p4est_quadrant_array_push (sc_array_t * array)
 {
+  p4est_quadrant_t *q;
+
   P4EST_ASSERT (array->elem_size == sizeof (p4est_quadrant_t));
 
-  return (p4est_quadrant_t *) sc_array_push (array);
+  q = (p4est_quadrant_t *) sc_array_push (array);
+  P4EST_QUADRANT_INIT(q);
+  return q;
 }
 
 /** Call sc_mempool_alloc for a mempool creating quadrants. */
