@@ -36,6 +36,23 @@
 #include <zlib.h>
 #endif
 
+int
+p4est_bsearch_partition (p4est_gloidx_t target,
+                         const p4est_gloidx_t * gfq, int nmemb)
+{
+  size_t              res;
+
+  P4EST_ASSERT (nmemb > 0);
+  P4EST_ASSERT (gfq[0] <= target);
+  P4EST_ASSERT (target < gfq[nmemb]);
+
+  res = sc_bsearch_range (&target, gfq, (size_t) nmemb,
+                          sizeof (p4est_gloidx_t), p4est_gloidx_compare);
+  P4EST_ASSERT (res < (size_t) nmemb);
+
+  return (int) res;
+}
+
 void
 p4est_comm_parallel_env_assign (p4est_t * p4est, sc_MPI_Comm mpicomm)
 {
@@ -391,6 +408,23 @@ p4est_comm_global_partition (p4est_t * p4est, p4est_quadrant_t * first_quad)
   }
 }
 
+void
+p4est_comm_global_first_quadrant (p4est_gloidx_t global_num_quadrants,
+                                  int mpisize, p4est_gloidx_t * gfq)
+{
+  int                 i;
+
+  P4EST_ASSERT (gfq != NULL);
+  P4EST_ASSERT (mpisize >= 1);
+  P4EST_ASSERT (global_num_quadrants >= 0);
+
+  gfq[0] = 0;
+  for (i = 1; i < mpisize; ++i) {
+    gfq[i] = p4est_partition_cut_gloidx (global_num_quadrants, i, mpisize);
+  }
+  gfq[mpisize] = global_num_quadrants;
+}
+
 void
 p4est_comm_count_pertree (p4est_t * p4est, p4est_gloidx_t * pertree)
 {
@@ -562,19 +596,31 @@ p4est_comm_count_pertree (p4est_t * p4est, p4est_gloidx_t * pertree)
 }
 
 int
-p4est_comm_is_empty (p4est_t * p4est, int p)
+p4est_comm_is_empty (p4est_t *p4est, int p)
 {
-  const p4est_gloidx_t *gfq;
-
   P4EST_ASSERT (p4est != NULL);
-  P4EST_ASSERT (0 <= p && p < p4est->mpisize);
+  return p4est_comm_is_empty_gfq (p4est->global_first_quadrant,
+                                  p4est->mpisize, p);
+}
 
-  gfq = p4est->global_first_quadrant;
+int
+p4est_comm_is_empty_gfq (const p4est_gloidx_t *gfq, int num_procs, int p)
+{
   P4EST_ASSERT (gfq != NULL);
+  P4EST_ASSERT (0 <= p && p < num_procs);
 
   return gfq[p] == gfq[p + 1];
 }
 
+int
+p4est_comm_is_empty_gfp (const p4est_quadrant_t *gfp, int num_procs, int p)
+{
+  P4EST_ASSERT (gfp != NULL);
+  P4EST_ASSERT (0 <= p && p < num_procs);
+
+  return p4est_quadrant_is_equal_piggy (&gfp[p], &gfp[p + 1]);
+}
+
 int
 p4est_comm_is_contained (p4est_t * p4est, p4est_locidx_t which_tree,
                          const p4est_quadrant_t * q, int rank)
@@ -623,22 +669,33 @@ p4est_comm_is_contained (p4est_t * p4est, p4est_locidx_t which_tree,
 }
 
 int
-p4est_comm_is_owner (p4est_t * p4est, p4est_locidx_t which_tree,
-                     const p4est_quadrant_t * q, int rank)
+p4est_comm_is_owner (p4est_t *p4est, p4est_locidx_t which_tree,
+                     const p4est_quadrant_t *q, int rank)
+{
+  P4EST_ASSERT (p4est != NULL);
+  P4EST_ASSERT (p4est->connectivity != NULL);
+
+  return p4est_comm_is_owner_gfp
+    (p4est->global_first_position, p4est->mpisize,
+     p4est->connectivity->num_trees, which_tree, q, rank);
+}
+
+int                 p4est_comm_is_owner_gfp
+  (const p4est_quadrant_t *gfp, int num_procs,
+   p4est_topidx_t num_trees, p4est_locidx_t which_tree,
+   const p4est_quadrant_t *q, int rank)
 {
   p4est_topidx_t      ctree;
   const p4est_quadrant_t *cur;
 
-  P4EST_ASSERT (p4est != NULL && p4est->connectivity != NULL);
-  P4EST_ASSERT (p4est->global_first_position != NULL);
-  P4EST_ASSERT (0 <= which_tree &&
-                which_tree < p4est->connectivity->num_trees);
+  P4EST_ASSERT (gfp != NULL);
+  P4EST_ASSERT (0 <= which_tree && which_tree < num_trees);
   P4EST_ASSERT (q != NULL);
-  P4EST_ASSERT (0 <= rank && rank < p4est->mpisize);
+  P4EST_ASSERT (0 <= rank && rank < num_procs);
   P4EST_ASSERT (p4est_quadrant_is_node (q, 1) || p4est_quadrant_is_valid (q));
 
   /* check whether q begins on a lower processor than rank */
-  cur = &p4est->global_first_position[rank];
+  cur = &gfp[rank];
   P4EST_ASSERT (cur->level == P4EST_QMAXLEVEL);
   ctree = cur->p.which_tree;
   if (which_tree < ctree ||
@@ -654,7 +711,7 @@ p4est_comm_is_owner (p4est_t * p4est, p4est_locidx_t which_tree,
 
   /* check whether q lies fully on a higher processor than rank */
   ++cur;
-  P4EST_ASSERT (cur == &p4est->global_first_position[rank + 1]);
+  P4EST_ASSERT (cur == &gfp[rank + 1]);
   P4EST_ASSERT (cur->level == P4EST_QMAXLEVEL);
   ctree = cur->p.which_tree;
   if (which_tree > ctree ||
@@ -885,29 +942,22 @@ p4est_comm_checksum (p4est_t * p4est, unsigned local_crc, size_t local_bytes)
 #ifdef P4EST_ENABLE_MPI
   int                 mpiret;
   int                 p;
-  uint64_t            send[2];
-  uint64_t           *gather;
+  long long           send[2];
+  long long          *gather;
 
-  send[0] = (uint64_t) local_crc;
-  send[1] = (uint64_t) local_bytes;
-  gather = NULL;
-  if (p4est->mpirank == 0) {
-    gather = P4EST_ALLOC (uint64_t, 2 * p4est->mpisize);
-  }
-  mpiret = sc_MPI_Gather (send, 2, sc_MPI_LONG_LONG_INT,
-                          gather, 2, sc_MPI_LONG_LONG_INT, 0, p4est->mpicomm);
+  send[0] = (long long) local_crc;
+  send[1] = (long long) local_bytes;
+  gather = P4EST_ALLOC (long long, 2 * p4est->mpisize);
+  mpiret = sc_MPI_Allgather (send, 2, sc_MPI_LONG_LONG_INT,
+                             gather, 2, sc_MPI_LONG_LONG_INT, p4est->mpicomm);
   SC_CHECK_MPI (mpiret);
 
-  if (p4est->mpirank == 0) {
-    for (p = 1; p < p4est->mpisize; ++p) {
-      crc = adler32_combine (crc, (uLong) gather[2 * p + 0],
-                             (z_off_t) gather[2 * p + 1]);
-    }
-    P4EST_FREE (gather);
-  }
-  else {
-    crc = 0;
+  crc = (uLong) gather[0];
+  for (p = 1; p < p4est->mpisize; ++p) {
+    crc = adler32_combine (crc, (uLong) gather[2 * p + 0],
+                           (z_off_t) gather[2 * p + 1]);
   }
+  P4EST_FREE (gather);
 #endif /* P4EST_ENABLE_MPI */
 
   return (unsigned) crc;
@@ -958,26 +1008,6 @@ p4est_transfer_assign_comm (const p4est_gloidx_t * dest_gfq,
                 src_gfq[*mpirank + 1] <= src_gfq[*mpisize]);
 }
 
-/** Given target, find index p such that gfq[p] <= target < gfq[p + 1].
- * \param [in] nmemb    Number of entries in array MINUS ONE.
- */
-static int
-p4est_bsearch_partition (p4est_gloidx_t target,
-                         const p4est_gloidx_t * gfq, int nmemb)
-{
-  size_t              res;
-
-  P4EST_ASSERT (nmemb > 0);
-  P4EST_ASSERT (gfq[0] <= target);
-  P4EST_ASSERT (target < gfq[nmemb]);
-
-  res = sc_bsearch_range (&target, gfq, (size_t) nmemb,
-                          sizeof (p4est_gloidx_t), p4est_gloidx_compare);
-  P4EST_ASSERT (res < (size_t) nmemb);
-
-  return (int) res;
-}
-
 p4est_transfer_context_t *
 p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
                             const p4est_gloidx_t * src_gfq,
@@ -1020,10 +1050,14 @@ p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
 
   /* figure out subset of processes to receive from */
   if (dest_begin < dest_end) {
+    P4EST_ASSERT (dest_data != NULL);
+
     /* our process as the receiver is not empty */
     first_sender = p4est_bsearch_partition (dest_begin, src_gfq, mpisize);
     P4EST_ASSERT (0 <= first_sender && first_sender < mpisize);
-    last_sender = p4est_bsearch_partition (dest_end - 1, src_gfq, mpisize);
+    last_sender =
+      p4est_bsearch_partition (dest_end - 1, &src_gfq[first_sender],
+                               mpisize - first_sender) + first_sender;
     P4EST_ASSERT (first_sender <= last_sender && last_sender < mpisize);
     tc->num_senders = last_sender - first_sender + 1;
     P4EST_ASSERT (tc->num_senders > 0);
@@ -1073,10 +1107,14 @@ p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
 
   /* figure out subset of processes to send to */
   if (src_begin < src_end) {
+    P4EST_ASSERT (src_data != NULL);
+
     /* our process as the sender is not empty */
     first_receiver = p4est_bsearch_partition (src_begin, dest_gfq, mpisize);
     P4EST_ASSERT (0 <= first_receiver && first_receiver < mpisize);
-    last_receiver = p4est_bsearch_partition (src_end - 1, dest_gfq, mpisize);
+    last_receiver =
+      p4est_bsearch_partition (src_end - 1, &dest_gfq[first_receiver],
+                               mpisize - first_receiver) + first_receiver;
     P4EST_ASSERT (first_receiver <= last_receiver && last_receiver < mpisize);
     tc->num_receivers = last_receiver - first_receiver + 1;
     P4EST_ASSERT (tc->num_receivers > 0);
@@ -1135,7 +1173,7 @@ p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
   return tc;
 }
 
-static void
+void
 p4est_transfer_end (p4est_transfer_context_t * tc)
 {
   int                 mpiret;
@@ -1164,7 +1202,7 @@ void
 p4est_transfer_fixed_end (p4est_transfer_context_t * tc)
 {
   P4EST_ASSERT (tc != NULL);
-  P4EST_ASSERT (!tc->variable);
+  P4EST_ASSERT (tc->variable == 0);
 
   p4est_transfer_end (tc);
 }
@@ -1181,21 +1219,25 @@ p4est_transfer_custom (const p4est_gloidx_t * dest_gfq,
   tc = p4est_transfer_custom_begin (dest_gfq, src_gfq, mpicomm, tag,
                                     dest_data, dest_sizes,
                                     src_data, src_sizes);
-  p4est_transfer_end (tc);
+  p4est_transfer_custom_end (tc);
 }
 
-p4est_transfer_context_t *
-p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
-                             const p4est_gloidx_t * src_gfq,
-                             sc_MPI_Comm mpicomm, int tag,
-                             void *dest_data, const int *dest_sizes,
-                             const void *src_data, const int *src_sizes)
+static p4est_transfer_context_t *
+p4est_transfer_begin (const p4est_gloidx_t * dest_gfq,
+                      const p4est_gloidx_t * src_gfq,
+                      sc_MPI_Comm mpicomm, int tag,
+                      void *dest_data, const int *dest_sizes,
+                      const void *src_data, const int *src_sizes,
+                      size_t item_size, int variable)
 {
   p4est_transfer_context_t *tc;
   int                 mpiret;
   int                 mpisize, mpirank;
   int                 q;
   int                 first_sender, last_sender;
+#ifdef P4EST_ENABLE_DEBUG
+  int                 old_last_sender, old_last_receiver;
+#endif
   int                 first_receiver, last_receiver;
   int                 i, ilen;
   const int          *rs;
@@ -1207,12 +1249,18 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
   p4est_gloidx_t      gbegin, gend;
   sc_MPI_Request     *rq;
 
-  P4EST_ASSERT (dest_sizes != NULL);
-  P4EST_ASSERT (src_sizes != NULL);
+  /* consistency of internal helper function */
+  P4EST_ASSERT (variable == 1 || variable == 2);
+  P4EST_ASSERT ((variable == 1 && item_size == 1) || variable == 2);
 
   /* setup context structure */
   tc = P4EST_ALLOC_ZERO (p4est_transfer_context_t, 1);
-  tc->variable = 1;
+  tc->variable = variable;
+
+  /* there is nothing to do when there is no data */
+  if (item_size == 0) {
+    return tc;
+  }
 
   /* grab local partition information */
   p4est_transfer_assign_comm (dest_gfq, src_gfq, mpicomm, &mpisize, &mpirank);
@@ -1227,10 +1275,19 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
 
   /* figure out subset of processes to receive from */
   if (dest_begin < dest_end) {
+    P4EST_ASSERT (dest_sizes != NULL);
+
     /* our process as the receiver is not empty */
     first_sender = p4est_bsearch_partition (dest_begin, src_gfq, mpisize);
     P4EST_ASSERT (0 <= first_sender && first_sender < mpisize);
-    last_sender = p4est_bsearch_partition (dest_end - 1, src_gfq, mpisize);
+    last_sender =
+      p4est_bsearch_partition (dest_end - 1, &src_gfq[first_sender],
+                               mpisize - first_sender) + first_sender;
+#ifdef P4EST_ENABLE_DEBUG
+    old_last_sender =
+      p4est_bsearch_partition (dest_end - 1, src_gfq, mpisize);
+    P4EST_ASSERT (last_sender == old_last_sender);
+#endif
     P4EST_ASSERT (first_sender <= last_sender && last_sender < mpisize);
     tc->num_senders = last_sender - first_sender + 1;
     P4EST_ASSERT (tc->num_senders > 0);
@@ -1255,7 +1312,7 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
       byte_len = 0;
       ilen = (int) (gend - gbegin);
       for (i = 0; i < ilen; ++i) {
-        byte_len += *rs++;
+        byte_len += item_size * *rs++;
       }
 
       /* choose how to treat the sender process */
@@ -1264,6 +1321,7 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
         *rq++ = sc_MPI_REQUEST_NULL;
       }
       else {
+        P4EST_ASSERT (dest_data != NULL);
         if (q == mpirank) {
           /* on the same rank we remember pointers for memcpy */
           cp_len = byte_len;
@@ -1284,10 +1342,19 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
 
   /* figure out subset of processes to send to */
   if (src_begin < src_end) {
+    P4EST_ASSERT (src_sizes != NULL);
+
     /* our process as the sender is not empty */
     first_receiver = p4est_bsearch_partition (src_begin, dest_gfq, mpisize);
     P4EST_ASSERT (0 <= first_receiver && first_receiver < mpisize);
-    last_receiver = p4est_bsearch_partition (src_end - 1, dest_gfq, mpisize);
+    last_receiver =
+      p4est_bsearch_partition (src_end - 1, &dest_gfq[first_receiver],
+                               mpisize - first_receiver) + first_receiver;
+#ifdef P4EST_ENABLE_DEBUG
+    old_last_receiver =
+      p4est_bsearch_partition (src_end - 1, dest_gfq, mpisize);
+    P4EST_ASSERT (last_receiver == old_last_receiver);
+#endif
     P4EST_ASSERT (first_receiver <= last_receiver && last_receiver < mpisize);
     tc->num_receivers = last_receiver - first_receiver + 1;
     P4EST_ASSERT (tc->num_receivers > 0);
@@ -1312,7 +1379,7 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
       byte_len = 0;
       ilen = (int) (gend - gbegin);
       for (i = 0; i < ilen; ++i) {
-        byte_len += *rs++;
+        byte_len += item_size * *rs++;
       }
 
       /* choose how to treat the receiver process */
@@ -1321,6 +1388,7 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
         *rq++ = sc_MPI_REQUEST_NULL;
       }
       else {
+        P4EST_ASSERT (src_data != NULL);
         if (q == mpirank) {
           /* on the same rank we remember pointers for memcpy */
           P4EST_ASSERT (cp_len == byte_len);
@@ -1350,11 +1418,61 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
   return tc;
 }
 
+p4est_transfer_context_t *
+p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
+                             const p4est_gloidx_t * src_gfq,
+                             sc_MPI_Comm mpicomm, int tag,
+                             void *dest_data, const int *dest_sizes,
+                             const void *src_data, const int *src_sizes)
+{
+  return p4est_transfer_begin
+    (dest_gfq, src_gfq, mpicomm, tag,
+     dest_data, dest_sizes, src_data, src_sizes, 1, 1);
+}
+
+p4est_transfer_context_t *
+p4est_transfer_items_begin (const p4est_gloidx_t * dest_gfq,
+                            const p4est_gloidx_t * src_gfq,
+                            sc_MPI_Comm mpicomm, int tag,
+                            void *dest_data, const int *dest_counts,
+                            const void *src_data, const int *src_counts,
+                            size_t item_size)
+{
+  return p4est_transfer_begin
+    (dest_gfq, src_gfq, mpicomm, tag,
+     dest_data, dest_counts, src_data, src_counts, item_size, 2);
+}
+
 void
 p4est_transfer_custom_end (p4est_transfer_context_t * tc)
 {
   P4EST_ASSERT (tc != NULL);
-  P4EST_ASSERT (tc->variable);
+  P4EST_ASSERT (tc->variable == 1);
+
+  p4est_transfer_end (tc);
+}
+
+void
+p4est_transfer_items (const p4est_gloidx_t * dest_gfq,
+                      const p4est_gloidx_t * src_gfq,
+                      sc_MPI_Comm mpicomm, int tag,
+                      void *dest_data, const int *dest_sizes,
+                      const void *src_data, const int *src_sizes,
+                      size_t item_size)
+{
+  p4est_transfer_context_t *tc;
+
+  tc = p4est_transfer_items_begin (dest_gfq, src_gfq, mpicomm, tag,
+                                   dest_data, dest_sizes,
+                                   src_data, src_sizes, item_size);
+  p4est_transfer_items_end (tc);
+}
+
+void
+p4est_transfer_items_end (p4est_transfer_context_t * tc)
+{
+  P4EST_ASSERT (tc != NULL);
+  P4EST_ASSERT (tc->variable == 2);
 
   p4est_transfer_end (tc);
 }
