@@ -44,7 +44,7 @@
  * Please see the documentation of \ref p4est_connectivity_t for the exact
  * encoding convention.
  *
- * We provide various predefined connectivitys by dedicated constructors,
+ * We provide various predefined connectivities by dedicated constructors,
  * such as
  *
  *  * \ref p4est_connectivity_new_unitsquare for the unit square,
@@ -218,6 +218,25 @@ typedef struct p4est_connectivity
 }
 p4est_connectivity_t;
 
+/** Management information for a connectivity shared by MPI3. */
+typedef struct p4est_connectivity_shared
+{
+  /** The members of this connectivity are MPI3 shared windows. */
+  p4est_connectivity_t *conn;
+#ifdef P4EST_ENABLE_MPIWINSHARED
+  MPI_Win             win_vertices;
+  MPI_Win             win_tree_to_vertex;
+  MPI_Win             win_tree_to_attr;
+  MPI_Win             win_tree_to_tree;
+  MPI_Win             win_tree_to_face;
+  MPI_Win             win_tree_to_corner;
+  MPI_Win             win_ctt_offset;
+  MPI_Win             win_corner_to_tree;
+  MPI_Win             win_corner_to_corner;
+#endif
+}
+p4est_connectivity_shared_t;
+
 /** Calculate memory usage of a connectivity structure.
  * \param [in] conn   Connectivity structure.
  * \return            Memory used in bytes.
@@ -306,15 +325,48 @@ void                p4est_connectivity_get_neighbor_transforms
 
 /* *INDENT-ON* */
 
+/** Determine the owning tree for a coordinate and transform it there.
+ *
+ * On a boundary between trees, different coordinate systems meet.
+ * A coordinate on a tree boundary face or corner generated from the
+ * perspective of a specific tree may be transformed into any other touching
+ * tree's coordinate system and still refer to the same point in the mesh.
+ *
+ * To uniquely identify a coordinate, this function identifies the lowest
+ * numbered tree touching this coordinate and transforms the coordinates
+ * into that system.  The result can be used e. g. in topology hash tables.
+ *
+ * \param [in] conn     A valid connectivity.
+ * \param [in] treeid   The original tree index for this coordinate tuple.
+ * \param [in] coords   A valid coordinate 2-tuple relative to \a treeid.
+ * \param [out] treeid_out   The lowest tree index touching the coordinate.
+ * \param [out] coords_out   The input coordinates, if necessary after
+ *                           transformation into the system of the lowest
+ *                           numbered tree, returned in \a treeid_out.
+ */
+void                p4est_connectivity_coordinates_canonicalize
+  (p4est_connectivity_t *conn,
+   p4est_topidx_t treeid, const p4est_qcoord_t coords[],
+   p4est_topidx_t *treeid_out, p4est_qcoord_t coords_out[]);
+
+/** Store the boundary point of the volume in [0, P4EST_INSUL). */
+extern const int    p4est_volume_point;
+
 /** Store the corner numbers 0..4 for each tree face. */
 extern const int    p4est_face_corners[4][2];
 
+/** For each face number, its boundary point in [0, P4EST_INSUL). */
+extern const int    p4est_face_points[4];
+
 /** Store the face numbers in the face neighbor's system. */
 extern const int    p4est_face_dual[4];
 
 /** Store the face numbers 0..3 for each tree corner. */
 extern const int    p4est_corner_faces[4][2];
 
+/** For each corner number, its boundary point in [0, P4EST_INSUL). */
+extern const int    p4est_corner_points[4];
+
 /** Store the face corner numbers for the faces touching a tree corner. */
 extern const int    p4est_corner_face_corners[4][4];
 
@@ -387,8 +439,24 @@ p4est_connectivity_t *p4est_connectivity_new_copy (p4est_topidx_t
                                                    const p4est_topidx_t * ctt,
                                                    const int8_t * ctc);
 
+/** Deep copy a connectivity structure.
+ * \param [in] input        Valid connectivity.
+ * \param [in] copy_attr    If true, we copy the tree attribute data.
+ *                          Otherwise, the result has empty attributes.
+ * \return              A connectivity equal to the first one except,
+ *                      depending on \a copy_attry, for its attributes.
+ */
+p4est_connectivity_t *p4est_connectivity_copy (p4est_connectivity_t *input,
+                                               int copy_attr);
+
 /** Broadcast a connectivity structure that exists only on one process to all.
- *  On the other processors, it will be allocated using p4est_connectivity_new.
+ *  On the other processors, it will be allocated using p4est_connectivity_new
+ *  and received.  This function is collective over the communicator passed.
+ *
+ *  This function may be called with a communicator that contains only one
+ *  rank of every shared memory node in preparation to subsequently calling
+ *  \ref p4est_connectivity_share with an intranode communicator.
+ *
  *  \param [in] conn_in For the root process the connectivity to be broadcast,
  *                      for the other processes it must be NULL.
  *  \param [in] root    The rank of the process that provides the connectivity.
@@ -407,6 +475,62 @@ p4est_connectivity_t *p4est_connectivity_bcast (p4est_connectivity_t *
 void                p4est_connectivity_destroy (p4est_connectivity_t *
                                                 connectivity);
 
+/** Take a connectivity on a single rank and share it with MPI3.
+ *  If MPI shared windows are not found at configure time, this function
+ *  calls \ref p4est_connectivity_bcast instead and wraps its result in the
+ *  result.  The function is collective over the communicator passed.
+ *
+ *  This function is only well defined for an intranode communicator.
+ *  Before calling it, the input connectivity may be made available on the
+ *  \a root rank using \ref p4est_connectivity_bcast with a surrounding
+ *  communicator that contains one root process of every node.
+ *
+ *  \param [in] conn_in For the root process a valid connectivity to be
+ *                      shared by MPI3.  This function takes ownership
+ *                      of this argument, so it must no longer be used.
+ *                      For all other processes it must be NULL.
+ *  \param [in] root    The rank of the process that provides the input
+ *                      connectivity.  Must be legal wrt. \a comm.
+ *  \param [in,out] comm    When configured with MPI3 enabled, this
+ *                      intranode communicator must permit MPI3 windows.
+ *  \return             The new connectivity object stores all data of the
+ *                      input \a conn in MPI3 shared windows.  Must be
+ *                      freed by \ref p4est_connectivity_shared_destroy.
+ */
+p4est_connectivity_shared_t *p4est_connectivity_share
+  (p4est_connectivity_t * conn_in, int root, sc_MPI_Comm comm);
+
+/** Take a connectivity on the world rank zero and share it globally.
+ * To this end, split the input communicator by node and broadcast
+ * the input connectivity among the first ranks of every node.
+ * In a second step, share it on each node from the first to all ranks.
+ *
+ * By the design of our wrappers for communicator splitting, this function
+ * also works with MPI but without type splitting available, and without MPI.
+ *
+ * \param [in] conn_in      Valid connectivity.  We take ownership of it.
+ *                          It must be accessed anymore after returning.
+ * \param [in] split_type   Should be sc_MPI_COMM_TYPE_SHARED or an
+ *                          implementation option such as to use the
+ *                          socket as relevant shared memory domain.
+ * \param [in] world_comm   Communicator encompassing all ranks on
+ *                          one or more shared memory nodes.
+ * \return                  Shared connectivity.  Free with \ref
+ *                          p4est_connectivity_shared_destroy.
+ */
+p4est_connectivity_shared_t *
+p4est_connectivity_mission (p4est_connectivity_t *conn_in,
+                            int split_type, sc_MPI_Comm world_comm);
+
+/** Destroy a shared connectivity structure.
+ * Call this eventually on the result of \ref p4est_connectivity_share
+ * or \ref p4est_connectivity_mission (which calls the former internally).
+ * \param [in] cshare       Valid shared connectivity structure;
+ *                          cf. \ref p4est_connectivity_share.
+ */
+void                p4est_connectivity_shared_destroy
+  (p4est_connectivity_shared_t *cshare);
+
 /** Allocate or free the attribute fields in a connectivity.
  * \param [in,out] conn         The conn->*_to_attr fields must either be NULL
  *                              or previously be allocated by this function.
