@@ -194,6 +194,9 @@ p4est_nodes_new_local (p4est_t * p4est)
               /* can't check for quadrants larger than the root */
               if (q->level == 0 && rlev < 0)
                 continue;
+              /* can't check for quadrants deeper than maxlevel */
+              if (q->level == P4EST_QMAXLEVEL && rlev > 0)
+                continue;
               /* can't check for quadrants larger unless child id
                * and corner line up
                */
@@ -274,7 +277,7 @@ p4est_node_canonicalize (p4est_t * p4est, p4est_topidx_t treeid,
   int                 contacts, face, corner;
   int                 ftransform[P4EST_FTRANSFORM];
   size_t              ctreez;
-  p4est_topidx_t      ntreeid, ntreeid2, lowest;
+  p4est_topidx_t      ntreeid, lowest;
   p4est_quadrant_t    tmpq, o;
 #ifdef P4_TO_P8
   int                 edge;
@@ -336,8 +339,8 @@ p4est_node_canonicalize (p4est_t * p4est, p4est_topidx_t treeid,
       continue;
     }
     /* Transform the node into the other tree's coordinates */
-    ntreeid2 = p4est_find_face_transform (conn, treeid, face, ftransform);
-    P4EST_ASSERT (ntreeid2 == ntreeid);
+    P4EST_EXECUTE_ASSERT_TOPIDX
+      (p4est_find_face_transform (conn, treeid, face, ftransform), ntreeid);
     p4est_quadrant_transform_face (n, &o, ftransform);
     if (ntreeid < lowest) {
       /* we have found a new owning tree */
@@ -446,12 +449,9 @@ endfunction:
 static int
 p4est_nodes_foreach (void **item, const void *u)
 {
-  const sc_hash_array_data_t *internal_data =
-    (const sc_hash_array_data_t *) u;
-  const p4est_locidx_t *new_node_number =
-    (const p4est_locidx_t *) internal_data->user_data;
+  const p4est_locidx_t *new_node_number = (const p4est_locidx_t *) u;
 
-  *item = (void *) (long) new_node_number[(long) *item];
+  *item = (void *) (size_t) new_node_number[(size_t) *item];
 
   return 1;
 }
@@ -729,10 +729,10 @@ p4est_nodes_new (p4est_t * p4est, p4est_ghost_t * ghost)
   }
 
   /* Re-synchronize hash array and local nodes */
-  save_user_data = indep_nodes->internal_data.user_data;
-  indep_nodes->internal_data.user_data = new_node_number;
-  sc_hash_foreach (indep_nodes->h, p4est_nodes_foreach);
-  indep_nodes->internal_data.user_data = save_user_data;
+  save_user_data = indep_nodes->user_data;
+  indep_nodes->user_data = new_node_number;
+  sc_hash_array_foreach (indep_nodes, p4est_nodes_foreach);
+  indep_nodes->user_data = save_user_data;
   for (il = 0; il < num_local_nodes; ++il) {
     P4EST_ASSERT (local_nodes[il] >= 0 && local_nodes[il] < num_indep_nodes);
     local_nodes[il] = new_node_number[local_nodes[il]];
@@ -882,7 +882,7 @@ p4est_nodes_new (p4est_t * p4est, p4est_ghost_t * ghost)
     k = probe_status.MPI_SOURCE;
     peer = peers + k;
     P4EST_ASSERT (k != rank && peer->expect_query);
-    mpiret = MPI_Get_count (&probe_status, MPI_BYTE, &byte_count);
+    mpiret = sc_MPI_Get_count (&probe_status, MPI_BYTE, &byte_count);
     SC_CHECK_MPI (mpiret);
     P4EST_ASSERT (byte_count % first_size == 0);
     elem_count = byte_count / (int) first_size;
@@ -1166,7 +1166,7 @@ p4est_nodes_new (p4est_t * p4est, p4est_ghost_t * ghost)
     k = probe_status.MPI_SOURCE;
     peer = peers + k;
     P4EST_ASSERT (k != rank && peer->expect_reply);
-    mpiret = MPI_Get_count (&probe_status, MPI_BYTE, &byte_count);
+    mpiret = sc_MPI_Get_count (&probe_status, MPI_BYTE, &byte_count);
     SC_CHECK_MPI (mpiret);
     sc_array_resize (&peer->recv_second, (size_t) byte_count);
     mpiret = MPI_Recv (peer->recv_second.array, byte_count, MPI_BYTE,
@@ -1260,9 +1260,9 @@ p4est_nodes_new (p4est_t * p4est, p4est_ghost_t * ghost)
 #ifdef P4EST_ENABLE_MPI
   /* Wait and close all send requests. */
   if (send_requests.elem_count > 0) {
-    mpiret = MPI_Waitall ((int) send_requests.elem_count,
-                          (MPI_Request *) send_requests.array,
-                          MPI_STATUSES_IGNORE);
+    mpiret = sc_MPI_Waitall ((int) send_requests.elem_count,
+                             (MPI_Request *) send_requests.array,
+                             MPI_STATUSES_IGNORE);
     SC_CHECK_MPI (mpiret);
   }
   nodes->shared_offsets = shared_offsets;
