@@ -63,6 +63,7 @@
 #define P4EST_LAST_OFFSET               P8EST_LAST_OFFSET
 #define P4EST_QUADRANT_INIT             P8EST_QUADRANT_INIT
 #define P4EST_LEAF_IS_FIRST_IN_TREE     P8EST_LEAF_IS_FIRST_IN_TREE
+#define P4EST_COORDINATES_IS_VALID      P8EST_COORDINATES_IS_VALID
 
 #ifdef P4EST_ENABLE_FILE_DEPRECATED
 
@@ -131,6 +132,7 @@
 #define p4est_connect_type_t            p8est_connect_type_t
 #define p4est_connectivity_encode_t     p8est_connectivity_encode_t
 #define p4est_connectivity_t            p8est_connectivity_t
+#define p4est_connectivity_shared_t     p8est_connectivity_shared_t
 #define p4est_corner_transform_t        p8est_corner_transform_t
 #define p4est_corner_info_t             p8est_corner_info_t
 #define p4est_neighbor_transform_t      p8est_neighbor_transform_t
@@ -156,6 +158,8 @@
 #define p4est_lnodes_buffer_t           p8est_lnodes_buffer_t
 #define p4est_iter_volume_t             p8est_iter_volume_t
 #define p4est_iter_volume_info_t        p8est_iter_volume_info_t
+#define p4est_iter_face_side_full_t     p8est_iter_face_side_full_t
+#define p4est_iter_face_side_hanging_t  p8est_iter_face_side_hanging_t
 #define p4est_iter_face_t               p8est_iter_face_t
 #define p4est_iter_face_info_t          p8est_iter_face_info_t
 #define p4est_iter_face_side_t          p8est_iter_face_side_t
@@ -183,11 +187,15 @@
 #define p4est_file_section_metadata_t   p8est_file_section_metadata_t
 
 /* redefine external variables */
+#define p4est_volume_point              p8est_volume_point
 #define p4est_face_corners              p8est_face_corners
+#define p4est_face_points               p8est_face_points
 #define p4est_face_dual                 p8est_face_dual
 #define p4est_corner_faces              p8est_corner_faces
 #define p4est_corner_face_corners       p8est_corner_face_corners
+#define p4est_corner_points             p8est_corner_points
 #define p4est_child_corner_faces        p8est_child_corner_faces
+#define p4est_lnodes_corner_hanging     p8est_lnodes_corner_hanging
 #define P4EST_DATA_UNINITIALIZED        P8EST_DATA_UNINITIALIZED
 
 /* functions in p4est_connectivity */
@@ -202,8 +210,13 @@
 #define p4est_connectivity_new_twotrees p8est_connectivity_new_twotrees
 #define p4est_connectivity_new_byname   p8est_connectivity_new_byname
 #define p4est_connectivity_new_copy     p8est_connectivity_new_copy
+#define p4est_connectivity_copy         p8est_connectivity_copy
 #define p4est_connectivity_bcast        p8est_connectivity_bcast
+#define p4est_connectivity_share        p8est_connectivity_share
+#define p4est_connectivity_mission      p8est_connectivity_mission
 #define p4est_connectivity_destroy      p8est_connectivity_destroy
+#define p4est_connectivity_shared_destroy       \
+        p8est_connectivity_shared_destroy
 #define p4est_connectivity_set_attr     p8est_connectivity_set_attr
 #define p4est_connectivity_is_valid     p8est_connectivity_is_valid
 #define p4est_connectivity_is_equal     p8est_connectivity_is_equal
@@ -218,12 +231,14 @@
 #define p4est_expand_face_transform     p8est_expand_face_transform
 #define p4est_find_face_transform       p8est_find_face_transform
 #define p4est_find_corner_transform     p8est_find_corner_transform
-#define p4est_neighbor_transform_coordinates \
+#define p4est_neighbor_transform_coordinates    \
         p8est_neighbor_transform_coordinates
-#define p4est_neighbor_transform_coordinates_reverse \
+#define p4est_neighbor_transform_coordinates_reverse    \
         p8est_neighbor_transform_coordinates_reverse
-#define p4est_connectivity_get_neighbor_transforms \
+#define p4est_connectivity_get_neighbor_transforms      \
         p8est_connectivity_get_neighbor_transforms
+#define p4est_connectivity_coordinates_canonicalize     \
+        p8est_connectivity_coordinates_canonicalize
 #define p4est_corner_array_index        p8est_corner_array_index
 #define p4est_connectivity_reorder      p8est_connectivity_reorder
 #define p4est_connectivity_reorder_newid                \
@@ -362,20 +377,26 @@
 #define p4est_quadrant_is_first_last    p8est_quadrant_is_first_last
 #define p4est_quadrant_enlarge_first    p8est_quadrant_enlarge_first
 #define p4est_quadrant_enlarge_last     p8est_quadrant_enlarge_last
+#define p4est_quadrant_root             p8est_quadrant_root
 #define p4est_quadrant_ancestor         p8est_quadrant_ancestor
 #define p4est_quadrant_parent           p8est_quadrant_parent
 #define p4est_quadrant_sibling          p8est_quadrant_sibling
 #define p4est_quadrant_child            p8est_quadrant_child
+#define p4est_quadrant_volume_coordinates       \
+        p8est_quadrant_volume_coordinates
 #define p4est_quadrant_face_neighbor    p8est_quadrant_face_neighbor
 #define p4est_quadrant_face_neighbor_extra p8est_quadrant_face_neighbor_extra
 #define p4est_quadrant_half_face_neighbors p8est_quadrant_half_face_neighbors
 #define p4est_quadrant_all_face_neighbors p8est_quadrant_all_face_neighbors
+#define p4est_quadrant_face_coordinates p8est_quadrant_face_coordinates
 #define p4est_quadrant_corner_neighbor  p8est_quadrant_corner_neighbor
 #define p4est_quadrant_corner_neighbor_extra    \
         p8est_quadrant_corner_neighbor_extra
 #define p4est_quadrant_half_corner_neighbor     \
         p8est_quadrant_half_corner_neighbor
 #define p4est_quadrant_corner_node      p8est_quadrant_corner_node
+#define p4est_quadrant_corner_coordinates       \
+        p8est_quadrant_corner_coordinates
 #define p4est_quadrant_children         p8est_quadrant_children
 #define p4est_quadrant_childrenv        p8est_quadrant_childrenv
 #define p4est_quadrant_childrenpv       p8est_quadrant_childrenpv
@@ -388,6 +409,8 @@
         p8est_coordinates_transform_face
 #define p4est_quadrant_transform_face   p8est_quadrant_transform_face
 #define p4est_quadrant_touches_corner   p8est_quadrant_touches_corner
+#define p4est_coordinates_transform_corner      \
+        p8est_coordinates_transform_corner
 #define p4est_quadrant_transform_corner p8est_quadrant_transform_corner
 #define p4est_quadrant_shift_corner     p8est_quadrant_shift_corner
 #define p4est_quadrant_linear_id        p8est_quadrant_linear_id
@@ -397,11 +420,11 @@
 #define p4est_quadrant_srand            p8est_quadrant_srand
 #define p4est_neighbor_transform_quadrant       \
         p8est_neighbor_transform_quadrant
-#define p4est_neighbor_transform_quadrant_reverse    \
+#define p4est_neighbor_transform_quadrant_reverse       \
         p8est_neighbor_transform_quadrant_reverse
-#define p4est_quadrant_is_ancestor_face          \
+#define p4est_quadrant_is_ancestor_face         \
         p8est_quadrant_is_ancestor_face
-#define p4est_quadrant_is_ancestor_corner        \
+#define p4est_quadrant_is_ancestor_corner       \
         p8est_quadrant_is_ancestor_corner
 
 /* functions in p4est_search */
@@ -512,9 +535,12 @@
 #endif /* P4EST_ENABLE_FILE_DEPRECATED */
 
 /* functions in p4est_geometry */
+#define p4est_geometry_transform_coordinates    \
+        p8est_geometry_transform_coordinates
 #define p4est_geometry_destroy          p8est_geometry_destroy
 #define p4est_geometry_new_connectivity p8est_geometry_new_connectivity
 #define p4est_geometry_connectivity_X   p8est_geometry_connectivity_X
+#define p4est_geometry_coordinates_lnodes p8est_geometry_coordinates_lnodes
 
 /* functions in p4est_vtk */
 #define p4est_vtk_context_new           p8est_vtk_context_new
