@@ -42,7 +42,8 @@
 #ifdef P4EST_HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#if defined P4EST_HAVE_WINSOCK2_H || defined _WIN32
+#ifdef _WIN32                   /* we assume Winsock2.h is always available, if _WIN32 */
+#define WIN32_LEAN_AND_MEAN     /* make sure Winsock.h is never included */
 #include <winsock2.h>
 #endif
 
