@@ -99,7 +99,8 @@ SC_EXTERN_C_BEGIN;
 /** p8est identification string */
 #define P8EST_STRING "p8est"
 
-/** Increase this number whenever the on-disk format for
+/** The revision number of the p8est ondisk file format.
+ * Increase this number whenever the on-disk format for
  * p8est_connectivity, p8est, or any other 3D data structure changes.
  * The format for reading and writing must be the same.
  */
@@ -252,6 +253,29 @@ typedef struct p8est_connectivity
 }
 p8est_connectivity_t;
 
+/** Management information for a connectivity shared by MPI3. */
+typedef struct p8est_connectivity_shared
+{
+  /** The members of this connectivity are MPI3 shared windows. */
+  p8est_connectivity_t *conn;
+#ifdef P4EST_ENABLE_MPIWINSHARED
+  MPI_Win             win_vertices;
+  MPI_Win             win_tree_to_vertex;
+  MPI_Win             win_tree_to_attr;
+  MPI_Win             win_tree_to_tree;
+  MPI_Win             win_tree_to_face;
+  MPI_Win             win_tree_to_edge;
+  MPI_Win             win_ett_offset;
+  MPI_Win             win_edge_to_tree;
+  MPI_Win             win_edge_to_edge;
+  MPI_Win             win_tree_to_corner;
+  MPI_Win             win_ctt_offset;
+  MPI_Win             win_corner_to_tree;
+  MPI_Win             win_corner_to_corner;
+#endif
+}
+p8est_connectivity_shared_t;
+
 /** Calculate memory usage of a connectivity structure.
  * \param [in] conn   Connectivity structure.
  * \return            Memory used in bytes.
@@ -360,12 +384,42 @@ void                p8est_connectivity_get_neighbor_transforms
 
 /* *INDENT-ON* */
 
+/** Determine the owning tree for a coordinate and transform it there.
+ *
+ * On a boundary between trees, different coordinate systems meet.
+ * A coordinate on a tree boundary face, edge, or corner generated from the
+ * perspective of a specific tree may be transformed into any other touching
+ * tree's coordinate system and still refer to the same point in the mesh.
+ *
+ * To uniquely identify a coordinate, this function identifies the lowest
+ * numbered tree touching this coordinate and transforms the coordinate into
+ * that system.  The result can be used e. g. in topology hash tables.
+ *
+ * \param [in] conn     A valid connectivity.
+ * \param [in] treeid   The original tree index for this coordinate tuple.
+ * \param [in] coords   A valid coordinate 2-tuple relative to \a treeid.
+ * \param [out] treeid_out   The lowest tree index touching the coordinate.
+ * \param [out] coords_out   The input coordinates, if necessary after
+ *                           transformation into the system of the lowest
+ *                           numbered tree, returned in \a treeid_out.
+ */
+void                p8est_connectivity_coordinates_canonicalize
+  (p8est_connectivity_t *conn,
+   p4est_topidx_t treeid, const p4est_qcoord_t coords[],
+   p4est_topidx_t *treeid_out, p4est_qcoord_t coords_out[]);
+
+/** Store the boundary point of the volume in [0, P8EST_INSUL). */
+extern const int    p8est_volume_point;
+
 /** Store the corner numbers 0..7 for each tree face. */
 extern const int    p8est_face_corners[6][4];
 
 /** Store the edge numbers 0..12 for each tree face. */
 extern const int    p8est_face_edges[6][4];
 
+/** For each face number, its boundary point in [0, P8EST_INSUL). */
+extern const int    p8est_face_points[6];
+
 /** Store the face numbers in the face neighbor's system. */
 extern const int    p8est_face_dual[6];
 
@@ -395,6 +449,9 @@ extern const int    p8est_edge_faces[12][2];
 /** Store the corner numbers 0..8 for each tree edge. */
 extern const int    p8est_edge_corners[12][2];
 
+/** For each edge number, its boundary point in [0, P8EST_INSUL). */
+extern const int    p8est_edge_points[12];
+
 /** Store the edge corner numbers 0..1 for the corners touching a tree edge
     or -1 if combination is invalid */
 extern const int    p8est_edge_edge_corners[12][8];
@@ -413,6 +470,9 @@ extern const int    p8est_corner_faces[8][3];
 /** Store the edge numbers 0..11 for each tree corner. */
 extern const int    p8est_corner_edges[8][3];
 
+/** For each corner number, its boundary point in [0, P8EST_INSUL). */
+extern const int    p8est_corner_points[8];
+
 /** Store the face corner numbers for the faces touching a tree corner.
     Is -1 for invalid combinations. */
 extern const int    p8est_corner_face_corners[8][6];
@@ -568,8 +628,24 @@ p8est_connectivity_t *p8est_connectivity_new_copy (p4est_topidx_t
                                                    const p4est_topidx_t * ctt,
                                                    const int8_t * ctc);
 
+/** Deep copy a connectivity structure.
+ * \param [in] input        Valid connectivity.
+ * \param [in] copy_attr    If true, we copy the tree attribute data.
+ *                          Otherwise, the result has empty attributes.
+ * \return              A connectivity equal to the first one except,
+ *                      depending on \a copy_attry, for its attributes.
+ */
+p8est_connectivity_t *p8est_connectivity_copy (p8est_connectivity_t *input,
+                                               int copy_attr);
+
 /** Broadcast a connectivity structure that exists only on one process to all.
- *  On the other processors, it will be allocated using p8est_connectivity_new.
+ *  On the other processors, it will be allocated using p8est_connectivity_new
+ *  and received.  This function is collective over the communicator passed.
+ *
+ *  This function may be called with a communicator that contains only one
+ *  rank of every shared memory node in preparation to subsequently calling
+ *  \ref p8est_connectivity_share with an intranode communicator.
+ *
  *  \param [in] conn_in For the root process the connectivity to be broadcast,
  *                      for the other processes it must be NULL.
  *  \param [in] root    The rank of the process that provides the connectivity.
@@ -588,6 +664,62 @@ p8est_connectivity_t *p8est_connectivity_bcast (p8est_connectivity_t *
 void                p8est_connectivity_destroy (p8est_connectivity_t *
                                                 connectivity);
 
+/** Take a connectivity on a single rank and share it with MPI3.
+ *  If MPI shared windows are not found at configure time, this function
+ *  calls \ref p8est_connectivity_bcast instead and wraps its result in the
+ *  result.  The function is collective over the communicator passed.
+ *
+ *  This function is only well defined for an intranode communicator.
+ *  Before calling it, the input connectivity may be made available on the
+ *  \a root rank using \ref p8est_connectivity_bcast with a surrounding
+ *  communicator that contains one root process of every node.
+ *
+ *  \param [in] conn_in For the root process a valid connectivity to be
+ *                      shared by MPI3.  This function takes ownership
+ *                      of this argument, so it must no longer be used.
+ *                      For all other processes it must be NULL.
+ *  \param [in] root    The rank of the process that provides the input
+ *                      connectivity.  Must be legal wrt. \a comm.
+ *  \param [in,out] comm    When configured with MPI3 enabled, this
+ *                      intranode communicator must permit MPI3 windows.
+ *  \return             The new connectivity object stores all data of the
+ *                      input \a conn in MPI3 shared windows.  Must be
+ *                      freed by \ref p8est_connectivity_shared_destroy.
+ */
+p8est_connectivity_shared_t *p8est_connectivity_share
+  (p8est_connectivity_t * conn_in, int root, sc_MPI_Comm comm);
+
+/** Take a connectivity on the world rank zero and share it globally.
+ * To this end, split the input communicator by node and broadcast
+ * the input connectivity among the first ranks of every node.
+ * In a second step, share it on each node from the first to all ranks.
+ *
+ * By the design of our wrappers for communicator splitting, this function
+ * also works with MPI but without type splitting available, and without MPI.
+ *
+ * \param [in] conn_in      Valid connectivity.  We take ownership of it.
+ *                          It must be accessed anymore after returning.
+ * \param [in] split_type   Should be sc_MPI_COMM_TYPE_SHARED or an
+ *                          implementation option such as to use the
+ *                          socket as relevant shared memory domain.
+ * \param [in] world_comm   Communicator encompassing all ranks on
+ *                          one or more shared memory nodes.
+ * \return                  Shared connectivity.  Free with \ref
+ *                          p8est_connectivity_shared_destroy.
+ */
+p8est_connectivity_shared_t *
+p8est_connectivity_mission (p8est_connectivity_t *conn_in,
+                            int split_type, sc_MPI_Comm world_comm);
+
+/** Destroy a shared connectivity structure.
+ * Call this eventually on the result of \ref p8est_connectivity_share
+ * or \ref p8est_connectivity_mission (which calls the former internally).
+ * \param [in] cshare       Valid shared connectivity structure;
+ *                          cf. \ref p8est_connectivity_share.
+ */
+void                p8est_connectivity_shared_destroy
+  (p8est_connectivity_shared_t *cshare);
+
 /** Allocate or free the attribute fields in a connectivity.
  * \param [in,out] conn         The conn->*_to_attr fields must either be NULL
  *                              or previously be allocated by this function.
@@ -702,6 +834,12 @@ p8est_connectivity_t *p8est_connectivity_new_twowrap (void);
  */
 p8est_connectivity_t *p8est_connectivity_new_rotcubes (void);
 
+/** Create a connectivity structure for two trees on top of each other.
+ * This connectivity is meant to be used with \ref p8est_geometry_new_pillow
+ * to map a spherical shell.
+ */
+p8est_connectivity_t *p8est_connectivity_new_pillow (void);
+
 /** An m by n by p array with periodicity in x, y, and z if
  * periodic_a, periodic_b, and periodic_c are true, respectively.
  */
@@ -963,7 +1101,7 @@ int                 p8est_connectivity_is_equivalent (p8est_connectivity_t *
 /** Return a pointer to a p8est_edge_transform_t array element. */
 /*@unused@*/
 static inline p8est_edge_transform_t *
-p8est_edge_array_index (sc_array_t * array, size_t it)
+p8est_edge_array_index (sc_array_t *array, size_t it)
 {
   P4EST_ASSERT (array->elem_size == sizeof (p8est_edge_transform_t));
   P4EST_ASSERT (it < array->elem_count);
@@ -975,7 +1113,7 @@ p8est_edge_array_index (sc_array_t * array, size_t it)
 /** Return a pointer to a p8est_corner_transform_t array element. */
 /*@unused@*/
 static inline p8est_corner_transform_t *
-p8est_corner_array_index (sc_array_t * array, size_t it)
+p8est_corner_array_index (sc_array_t *array, size_t it)
 {
   P4EST_ASSERT (array->elem_size == sizeof (p8est_corner_transform_t));
   P4EST_ASSERT (it < array->elem_count);
