@@ -32,7 +32,6 @@
 #ifndef P8EST_ITERATE_H
 #define P8EST_ITERATE_H
 
-#include <p8est.h>
 #include <p8est_ghost.h>
 
 SC_EXTERN_C_BEGIN;
@@ -63,6 +62,25 @@ p8est_iter_volume_info_t;
 typedef void        (*p8est_iter_volume_t) (p8est_iter_volume_info_t * info,
                                             void *user_data);
 
+/** Shortcut to access full face information */
+typedef struct p8est_iter_face_side_full
+{
+  int8_t              is_ghost; /**< boolean: local (0) or ghost */
+  p8est_quadrant_t   *quad;     /**< the actual quadrant */
+  p4est_locidx_t      quadid;   /**< index in tree or ghost array */
+
+}
+p8est_iter_face_side_full_t;
+
+/** Shortcut to access hanging face information */
+typedef struct p8est_iter_face_side_hanging
+{
+  int8_t              is_ghost[4];      /**< boolean: local (0) or ghost */
+  p8est_quadrant_t   *quad[4];          /**< the actual quadrant */
+  p4est_locidx_t      quadid[4];        /**< index in tree or ghost array */
+}
+p8est_iter_face_side_hanging_t;
+
 /** Information about one side of a face in the forest.  If a \a quad is local
  * (\a is_ghost is false), then its \a quadid indexes the tree's quadrant array;
  * otherwise, it indexes the ghosts array. If the face is hanging, then the
@@ -79,22 +97,11 @@ typedef struct p8est_iter_face_side
                                             four smaller quads (1) */
   union p8est_iter_face_side_data
   {
-    struct
-    {
-      int8_t              is_ghost;    /**< boolean: local (0) or ghost (1) */
-      p8est_quadrant_t   *quad;        /**< the actual quadrant */
-      p4est_locidx_t      quadid;      /**< index in tree or ghost array */
-    }
-    full; /**< if \a is_hanging = 0,
-               use is.full to access per-quadrant data */
-    struct
-    {
-      int8_t              is_ghost[4]; /**< boolean: local (0) or ghost (1) */
-      p8est_quadrant_t   *quad[4];     /**< the actual quadrant */
-      p4est_locidx_t      quadid[4];   /**< index in tree or ghost array */
-    }
-    hanging; /**< if \a is_hanging = 1,
-                  use is.hanging to access per-quadrant data */
+    /** if \a !is_hanging, use is.full to access per-quadrant data */
+    p8est_iter_face_side_full_t full;
+
+    /** if \a is_hanging, use is.hanging to access per-quadrant data */
+    p8est_iter_face_side_hanging_t hanging;
   }
   is;
 }
@@ -119,7 +126,7 @@ typedef struct p8est_iter_face_info
                                         other, as in the definition of
                                         p8est_connectivity_t */
   int8_t              tree_boundary; /**< boolean: interior face (0),
-                                          boundary face (1) */
+                                          tree boundary face (true) */
   sc_array_t          sides;    /* array of p8est_iter_face_side_t type */
 }
 p8est_iter_face_info_t;
@@ -137,6 +144,25 @@ p8est_iter_face_info_t;
 typedef void        (*p8est_iter_face_t) (p8est_iter_face_info_t * info,
                                           void *user_data);
 
+/** Shortcut to access full edge information */
+typedef struct p8est_iter_edge_side_full
+{
+  int8_t              is_ghost; /**< boolean: local (0) or ghost */
+  p8est_quadrant_t   *quad;     /**< the actual quadrant */
+  p4est_locidx_t      quadid;   /**< index in tree or ghost array */
+
+}
+p8est_iter_edge_side_full_t;
+
+/** Shortcut to access hanging edge information */
+typedef struct p8est_iter_edge_side_hanging
+{
+  int8_t              is_ghost[2];      /**< boolean: local (0) or ghost */
+  p8est_quadrant_t   *quad[2];          /**< the actual quadrant */
+  p4est_locidx_t      quadid[2];        /**< index in tree or ghost array */
+}
+p8est_iter_edge_side_hanging_t;
+
 /* The information that is available to the user-defined p8est_iter_edge_t
  * callback.
  *
@@ -165,23 +191,11 @@ typedef struct p8est_iter_edge_side
                                             two smaller quads (1) */
   union p8est_iter_edge_side_data
   {
-    struct
-    {
-      int8_t              is_ghost;    /**< boolean: local (0) or ghost (1) */
-      p8est_quadrant_t   *quad;        /**< the actual quadrant */
-      p4est_locidx_t      quadid;      /**< index in tree or ghost array */
-    }
-    full; /**< if \a is_hanging = 0,
-               use is.full to access per-quadrant data */
+    /** if \a !is_hanging, use is.full to access per-quadrant data */
+    p8est_iter_edge_side_full_t full;
 
-    struct
-    {
-      int8_t              is_ghost[2]; /**< boolean: local (0) or ghost (1) */
-      p8est_quadrant_t   *quad[2];     /**< the actual quadrant */
-      p4est_locidx_t      quadid[2];   /**< index in tree or ghost array */
-    }
-    hanging; /**< if \a is_hanging = 1,
-                  use is.hanging to access per-quadrant data */
+    /** if \a is_hanging, use is.hanging to access per-quadrant data */
+    p8est_iter_edge_side_hanging_t hanging;
   }
   is;
   int8_t              faces[2];
@@ -200,7 +214,7 @@ typedef struct p8est_iter_edge_info
   p8est_t            *p4est;
   p8est_ghost_t      *ghost_layer;
   int8_t              tree_boundary;  /**< boolean: interior face (0),
-                                           boundary face (1) */
+                                           tree boundary face (true) */
   sc_array_t          sides; /**< array of p8est_iter_edge_side_t type */
 }
 p8est_iter_edge_info_t;
@@ -240,7 +254,7 @@ typedef struct p8est_iter_corner_side
 }
 p8est_iter_corner_side_t;
 
-/** The information that is availalbe to the user-defined p8est_iter_corner_t
+/** The information that is available to the user-defined p8est_iter_corner_t
  * callback.
  *
  * If tree_boundary is false, the corner is on the interior of a tree.
@@ -254,7 +268,7 @@ typedef struct p8est_iter_corner_info
   p8est_t            *p4est;
   p8est_ghost_t      *ghost_layer;
   int8_t              tree_boundary; /**< boolean: interior face (0),
-                                           boundary face (1) */
+                                          tree boundary face (true) */
   sc_array_t          sides; /**< array of p8est_iter_corner_side_t type */
 }
 p8est_iter_corner_info_t;
