@@ -22,65 +22,9 @@
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
-#include <p8est_algorithms.h>
-#include <p8est_bits.h>
-#include <p8est_extended.h>
+/* we grab a few static functions from the 2D file */
 #include <p4est_to_p8est.h>
-
-static int
-refine_none (p4est_t * p4est, p4est_topidx_t which_tree, p4est_quadrant_t * q)
-{
-  return 0;
-}
-
-static int
-refine_some (p4est_t * p4est, p4est_topidx_t which_tree, p4est_quadrant_t * q)
-{
-  if (q->x < P4EST_QUADRANT_LEN (2)) {
-    return q->level <= 4;
-  }
-  else if (q->x < P4EST_QUADRANT_LEN (1)) {
-    return q->level <= 3;
-  }
-  else {
-    return q->level <= 2;
-  }
-}
-
-static int
-coarsen_none (p4est_t * p4est, p4est_topidx_t which_tree,
-              p4est_quadrant_t * q[])
-{
-  SC_CHECK_ABORT (p4est_quadrant_is_familypv (q), "Coarsen invocation");
-
-  return 0;
-}
-
-static int
-coarsen_some (p4est_t * p4est, p4est_topidx_t which_tree,
-              p4est_quadrant_t * q[])
-{
-  SC_CHECK_ABORT (p4est_quadrant_is_familypv (q), "Coarsen invocation");
-
-  if (q[0]->x < P4EST_QUADRANT_LEN (2)) {
-    return q[0]->level >= 2;
-  }
-  else if (q[0]->x < P4EST_QUADRANT_LEN (1)) {
-    return q[0]->level >= 3;
-  }
-  else {
-    return q[0]->level >= 4;
-  }
-}
-
-static int
-coarsen_all (p4est_t * p4est, p4est_topidx_t which_tree,
-             p4est_quadrant_t * q[])
-{
-  SC_CHECK_ABORT (p4est_quadrant_is_familypv (q), "Coarsen invocation");
-
-  return 1;
-}
+#include "test_quadrants2.c"
 
 static void
 check_linear_id (const p4est_quadrant_t * q1, const p4est_quadrant_t * q2)
@@ -88,20 +32,27 @@ check_linear_id (const p4est_quadrant_t * q1, const p4est_quadrant_t * q2)
   int                 l;
   int                 comp = p4est_quadrant_compare (q1, q2);
   int                 level = (int) SC_MIN (q1->level, q2->level);
-  uint64_t            id1 = p4est_quadrant_linear_id (q1, level);
-  uint64_t            id2 = p4est_quadrant_linear_id (q2, level);
+  p4est_lid_t         id1;
+  p4est_lid_t         id2;
   p4est_quadrant_t    quad, par, anc;
 
+  p4est_quadrant_linear_id_ext128 (q1, level, &id1);
+  p4est_quadrant_linear_id_ext128 (q2, level, &id2);
+
   /* test linear id */
   if (p4est_quadrant_is_ancestor (q1, q2)) {
-    SC_CHECK_ABORT (id1 == id2 && comp < 0, "Ancestor 1");
+    SC_CHECK_ABORT (p4est_lid_is_equal (&id1, &id2)
+                    && comp < 0, "Ancestor 1");
   }
   else if (p4est_quadrant_is_ancestor (q2, q1)) {
-    SC_CHECK_ABORT (id1 == id2 && comp > 0, "Ancestor 2");
+    SC_CHECK_ABORT (p4est_lid_is_equal (&id1, &id2)
+                    && comp > 0, "Ancestor 2");
   }
   else {
-    SC_CHECK_ABORT ((comp == 0 && id1 == id2) || (comp < 0 && id1 < id2)
-                    || (comp > 0 && id1 > id2), "compare");
+    SC_CHECK_ABORT ((comp == 0 && p4est_lid_is_equal (&id1, &id2))
+                    || (comp < 0 && (p4est_lid_compare (&id1, &id2) < 0))
+                    || (comp > 0
+                        && (p4est_lid_compare (&id1, &id2) > 0)), "compare");
   }
 
   /* test ancestor and parent functions */
@@ -119,6 +70,45 @@ check_linear_id (const p4est_quadrant_t * q1, const p4est_quadrant_t * q2)
   }
 }
 
+static void
+check_successor_predecessor (const p4est_quadrant_t * q)
+{
+  p4est_quadrant_t    temp1, temp2;
+  p4est_lid_t         lid, successor_lid, one;
+
+  p4est_quadrant_linear_id_ext128 (q, q->level, &lid);
+  p4est_quadrant_successor (q, &temp1);
+  p4est_quadrant_linear_id_ext128 (&temp1, q->level, &successor_lid);
+  p4est_lid_set_one (&one);
+  p4est_lid_add_inplace (&lid, &one);
+  SC_CHECK_ABORT (p4est_lid_is_equal (&successor_lid, &lid), "successor");
+  p4est_quadrant_predecessor (&temp1, &temp2);
+  /* Check if predecessor inverts successor. */
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&temp2, q), "predecessor");
+}
+
+static void
+check_predecessor_successor (const p4est_quadrant_t * q)
+{
+  p4est_quadrant_t    temp1, temp2;
+  p4est_lid_t         lid, predecessor_lid, one;
+
+  p4est_quadrant_linear_id_ext128 (q, q->level, &lid);
+  p4est_quadrant_predecessor (q, &temp1);
+  p4est_quadrant_linear_id_ext128 (&temp1, q->level, &predecessor_lid);
+  p4est_lid_set_one (&one);
+  p4est_lid_sub_inplace (&lid, &one);
+  SC_CHECK_ABORT (p4est_lid_is_equal (&predecessor_lid, &lid), "predecessor");
+  p4est_quadrant_successor (&temp1, &temp2);
+  /* Check if successor inverts predecessor. */
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&temp2, q), "successor");
+}
+
+#define NEG_ONE_MAXL (~((((p4est_qcoord_t) 1) << P8EST_MAXLEVEL) - 1))
+#define NEG_ONE_MAXLM1 (~((((p4est_qcoord_t) 1) << (P8EST_MAXLEVEL - 1)) - 1))
+#define NEG_ONE_MAXLP1 \
+  (NEG_ONE_MAXL & ~(((p4est_qcoord_t) 1) << P8EST_MAXLEVEL))
+
 int
 main (int argc, char **argv)
 {
@@ -138,13 +128,16 @@ main (int argc, char **argv)
   p4est_quadrant_t    r, s;
   p4est_quadrant_t    c0, c1, c2, c3, c4, c5, c6, c7;
   p4est_quadrant_t    cv[P4EST_CHILDREN], *cp[P4EST_CHILDREN];
-  p4est_quadrant_t    A, B, C, D, E, F, G, H, I, P, Q;
+  p4est_quadrant_t    A, B, C, D, E, F, G, H, I, P, Q, R, S;
   p4est_quadrant_t    a, f, g, h;
-  uint64_t            Aid, Fid;
+  uint64_t            Aid;
+  p4est_lid_t         Fid;
   const int           indices[27] = { 0, 1, 2, 3, 4, 5, 6, 7,
     7, 9, 11, 13, 18, 19, 22, 23, 27, 31,
     36, 37, 38, 39, 45, 47, 54, 55, 63
   };
+  p4est_lid_t         id;
+  sc_rand_state_t     state;
 
   /* initialize MPI */
   mpiret = sc_MPI_Init (&argc, &argv);
@@ -170,6 +163,9 @@ main (int argc, char **argv)
   for (iz = 0; iz < t1->quadrants.elem_count; ++iz) {
     q1 = p4est_quadrant_array_index (&t1->quadrants, iz);
 
+    /* test coordinates of all boundary objects */
+    check_coordinates (q1);
+
     /* test the index conversion */
     index1 = p4est_quadrant_linear_id (q1, (int) q1->level);
     p4est_quadrant_set_morton (&r, (int) q1->level, index1);
@@ -244,7 +240,7 @@ main (int argc, char **argv)
       /* test the comparison function */
       SC_CHECK_ABORT (p4est_quadrant_compare (q1, q2) ==
                       -p4est_quadrant_compare (q2, q1), "compare");
-      SC_CHECK_ABORT ((p4est_quadrant_compare (q1, q2) == 0) ==
+      SC_CHECK_ABORT ((!p4est_quadrant_compare (q1, q2)) ==
                       p4est_quadrant_is_equal (q1, q2), "is_equal");
 
       /* test the descriptive versions of functions */
@@ -277,7 +273,7 @@ main (int argc, char **argv)
       /* test the comparison function */
       SC_CHECK_ABORT (p4est_quadrant_compare (q1, q2) ==
                       -p4est_quadrant_compare (q2, q1), "compare");
-      SC_CHECK_ABORT ((p4est_quadrant_compare (q1, q2) == 0) ==
+      SC_CHECK_ABORT ((!p4est_quadrant_compare (q1, q2)) ==
                       p4est_quadrant_is_equal (q1, q2), "is_equal");
 
       /* test the descriptive versions of functions */
@@ -443,18 +439,19 @@ main (int argc, char **argv)
   P4EST_QUADRANT_INIT (&I);
   P4EST_QUADRANT_INIT (&P);
   P4EST_QUADRANT_INIT (&Q);
+  P4EST_QUADRANT_INIT (&R);
 
-  A.x = -qone << P4EST_MAXLEVEL;
-  A.y = -qone << P4EST_MAXLEVEL;
+  A.x = NEG_ONE_MAXL;
+  A.y = NEG_ONE_MAXL;
   A.z = 0;
   A.level = 0;
 
   B.x = qone << P4EST_MAXLEVEL;
-  B.y = -qone << P4EST_MAXLEVEL;
+  B.y = NEG_ONE_MAXL;
   B.z = 0;
   B.level = 0;
 
-  C.x = -qone << P4EST_MAXLEVEL;
+  C.x = NEG_ONE_MAXL;
   C.y = qone << P4EST_MAXLEVEL;
   C.z = 0;
   C.level = 0;
@@ -465,14 +462,14 @@ main (int argc, char **argv)
   D.level = 0;
 
   /* this one is outside the 3x3 box */
-  E.x = -qone << (P4EST_MAXLEVEL + 1);
+  E.x = NEG_ONE_MAXLP1;
   E.y = -qone;
   E.z = -qone;
   E.level = 0;
 
   F.x = P4EST_ROOT_LEN + (P4EST_ROOT_LEN - mh);
   F.y = P4EST_ROOT_LEN + (P4EST_ROOT_LEN - mh);
-  F.z = -qone << P4EST_MAXLEVEL;
+  F.z = NEG_ONE_MAXL;
   F.level = P4EST_QMAXLEVEL;
 
   G.x = -mh;
@@ -480,18 +477,18 @@ main (int argc, char **argv)
   G.z = -mh;
   G.level = P4EST_QMAXLEVEL;
 
-  H.x = -qone << (P4EST_MAXLEVEL - 1);
-  H.y = -qone << (P4EST_MAXLEVEL - 1);
+  H.x = NEG_ONE_MAXLM1;
+  H.y = NEG_ONE_MAXLM1;
   H.z = qone << (P4EST_MAXLEVEL - 1);
   H.level = 1;
 
-  I.x = -qone << P4EST_MAXLEVEL;
-  I.y = -qone << (P4EST_MAXLEVEL - 1);
+  I.x = NEG_ONE_MAXL;
+  I.y = NEG_ONE_MAXLM1;
   I.z = P4EST_ROOT_LEN + (P4EST_ROOT_LEN - mh);
   I.level = P4EST_QMAXLEVEL;
 
-  P.x = -qone << P4EST_MAXLEVEL;
-  P.y = -qone << (P4EST_MAXLEVEL - 1);
+  P.x = NEG_ONE_MAXL;
+  P.y = NEG_ONE_MAXLM1;
   P.z = qone << (P4EST_MAXLEVEL - 1);
   P.level = 1;
 
@@ -500,6 +497,22 @@ main (int argc, char **argv)
   Q.z = (qone << P4EST_MAXLEVEL) - 2 * mh;
   Q.level = P4EST_QMAXLEVEL - 1;
 
+  R.x = 2;
+  R.y = 2;
+  R.z = 2;
+  R.level = P4EST_QMAXLEVEL;
+  check_coordinates (&R);
+
+  p4est_lid_set_zero (&id);
+  p4est_lid_set_bit (&id, 70);
+  p4est_quadrant_set_morton_ext128 (&S, P4EST_QMAXLEVEL, &id);
+
+  p4est_quadrant_srand (&R, &state);
+  SC_CHECK_ABORT ((uint64_t) state == 7, "quadrant_srand");
+  p4est_quadrant_srand (&S, &state);
+  SC_CHECK_ABORT ((uint64_t) state == (id.high_bits ^ id.low_bits),
+                  "quadrant_srand");
+
   SC_CHECK_ABORT (p4est_quadrant_compare (&B, &F) < 0, "Comp 1");
   SC_CHECK_ABORT (p4est_quadrant_compare (&A, &G) < 0, "Comp 2");
   SC_CHECK_ABORT (p4est_quadrant_compare (&F, &G) < 0, "Comp 3");
@@ -568,84 +581,78 @@ main (int argc, char **argv)
   check_linear_id (&Q, &H);
   check_linear_id (&Q, &I);
 
-  SC_CHECK_ABORT (p4est_quadrant_is_extended (&A) == 1, "is_extended A");
-  SC_CHECK_ABORT (p4est_quadrant_is_extended (&B) == 1, "is_extended B");
-  SC_CHECK_ABORT (p4est_quadrant_is_extended (&C) == 1, "is_extended C");
-  SC_CHECK_ABORT (p4est_quadrant_is_extended (&D) == 1, "is_extended D");
-  SC_CHECK_ABORT (!p4est_quadrant_is_extended (&E) == 1, "!is_extended E");
-  SC_CHECK_ABORT (p4est_quadrant_is_extended (&F) == 1, "is_extended F");
-  SC_CHECK_ABORT (p4est_quadrant_is_extended (&G) == 1, "is_extended G");
+  SC_CHECK_ABORT (p4est_quadrant_is_extended (&A), "is_extended A");
+  SC_CHECK_ABORT (p4est_quadrant_is_extended (&B), "is_extended B");
+  SC_CHECK_ABORT (p4est_quadrant_is_extended (&C), "is_extended C");
+  SC_CHECK_ABORT (p4est_quadrant_is_extended (&D), "is_extended D");
+  SC_CHECK_ABORT (!p4est_quadrant_is_extended (&E), "!is_extended E");
+  SC_CHECK_ABORT (p4est_quadrant_is_extended (&F), "is_extended F");
+  SC_CHECK_ABORT (p4est_quadrant_is_extended (&G), "is_extended G");
 
-  SC_CHECK_ABORT (p4est_quadrant_compare (&A, &A) == 0, "compare");
+  SC_CHECK_ABORT (!p4est_quadrant_compare (&A, &A), "compare");
   SC_CHECK_ABORT (p4est_quadrant_compare (&A, &B) > 0, "compare");
   SC_CHECK_ABORT (p4est_quadrant_compare (&B, &A) < 0, "compare");
 
-  SC_CHECK_ABORT (p4est_quadrant_compare (&F, &F) == 0, "compare");
+  SC_CHECK_ABORT (!p4est_quadrant_compare (&F, &F), "compare");
   SC_CHECK_ABORT (p4est_quadrant_compare (&G, &F) > 0, "compare");
   SC_CHECK_ABORT (p4est_quadrant_compare (&F, &G) < 0, "compare");
 
   A.p.piggy1.which_tree = 0;
   B.p.piggy2.which_tree = 0;
-  SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&A, &A) == 0,
-                  "compare_piggy");
+  SC_CHECK_ABORT (!p4est_quadrant_compare_piggy (&A, &A), "compare_piggy");
   SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&A, &B) > 0, "compare_piggy");
   SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&B, &A) < 0, "compare_piggy");
 
   F.p.which_tree = 0;
   G.p.piggy1.which_tree = 0;
-  SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&F, &F) == 0,
-                  "compare_piggy");
+  SC_CHECK_ABORT (!p4est_quadrant_compare_piggy (&F, &F), "compare_piggy");
   SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&G, &F) > 0, "compare_piggy");
   SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&F, &G) < 0, "compare_piggy");
 
   F.p.piggy2.which_tree = (p4est_topidx_t) P4EST_TOPIDX_MAX - 3;
   G.p.which_tree = (p4est_topidx_t) P4EST_TOPIDX_MAX / 2;
-  SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&F, &F) == 0,
-                  "compare_piggy");
+  SC_CHECK_ABORT (!p4est_quadrant_compare_piggy (&F, &F), "compare_piggy");
   SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&G, &F) < 0, "compare_piggy");
   SC_CHECK_ABORT (p4est_quadrant_compare_piggy (&F, &G) > 0, "compare_piggy");
 
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &A) == 1, "is_equal");
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&F, &F) == 1, "is_equal");
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&G, &G) == 1, "is_equal");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &A), "is_equal");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&F, &F), "is_equal");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&G, &G), "is_equal");
 
-  SC_CHECK_ABORT (p4est_quadrant_is_sibling (&P, &H) == 1, "is_sibling");
-  SC_CHECK_ABORT (p4est_quadrant_is_sibling (&A, &H) == 0, "is_sibling");
-  SC_CHECK_ABORT (p4est_quadrant_is_sibling_D (&P, &H) == 1, "is_sibling_D");
-  SC_CHECK_ABORT (p4est_quadrant_is_sibling_D (&A, &H) == 0, "is_sibling_D");
+  SC_CHECK_ABORT (p4est_quadrant_is_sibling (&P, &H), "is_sibling");
+  SC_CHECK_ABORT (!p4est_quadrant_is_sibling (&A, &H), "is_sibling");
+  SC_CHECK_ABORT (p4est_quadrant_is_sibling_D (&P, &H), "is_sibling_D");
+  SC_CHECK_ABORT (!p4est_quadrant_is_sibling_D (&A, &H), "is_sibling_D");
 
-  SC_CHECK_ABORT (p4est_quadrant_is_parent (&A, &H) == 1, "is_parent");
-  SC_CHECK_ABORT (p4est_quadrant_is_parent (&H, &A) == 0, "is_parent");
-  SC_CHECK_ABORT (p4est_quadrant_is_parent (&A, &Q) == 0, "is_parent");
-  SC_CHECK_ABORT (p4est_quadrant_is_parent_D (&A, &H) == 1, "is_parent_D");
+  SC_CHECK_ABORT (p4est_quadrant_is_parent (&A, &H), "is_parent");
+  SC_CHECK_ABORT (!p4est_quadrant_is_parent (&H, &A), "is_parent");
+  SC_CHECK_ABORT (!p4est_quadrant_is_parent (&A, &Q), "is_parent");
+  SC_CHECK_ABORT (p4est_quadrant_is_parent_D (&A, &H), "is_parent_D");
 
-  SC_CHECK_ABORT (p4est_quadrant_is_ancestor (&A, &Q) == 1, "is_ancestor");
-  SC_CHECK_ABORT (p4est_quadrant_is_ancestor (&A, &A) == 0, "is_ancestor");
+  SC_CHECK_ABORT (p4est_quadrant_is_ancestor (&A, &Q), "is_ancestor");
+  SC_CHECK_ABORT (!p4est_quadrant_is_ancestor (&A, &A), "is_ancestor");
 
-  SC_CHECK_ABORT (p4est_quadrant_is_ancestor_D (&A, &P) == 1,
-                  "is_ancestor_D");
-  SC_CHECK_ABORT (p4est_quadrant_is_ancestor_D (&G, &G) == 0,
-                  "is_ancestor_D");
+  SC_CHECK_ABORT (p4est_quadrant_is_ancestor_D (&A, &P), "is_ancestor_D");
+  SC_CHECK_ABORT (!p4est_quadrant_is_ancestor_D (&G, &G), "is_ancestor_D");
 
-  /* SC_CHECK_ABORT (p4est_quadrant_is_next (&F, &E) == 1, "is_next"); */
-  SC_CHECK_ABORT (p4est_quadrant_is_next (&A, &H) == 0, "is_next");
-  /* SC_CHECK_ABORT (p4est_quadrant_is_next_D (&F, &E) == 1, "is_next_D"); */
-  SC_CHECK_ABORT (p4est_quadrant_is_next_D (&A, &H) == 0, "is_next_D");
+  /* SC_CHECK_ABORT (p4est_quadrant_is_next (&F, &E), "is_next"); */
+  SC_CHECK_ABORT (!p4est_quadrant_is_next (&A, &H), "is_next");
+  /* SC_CHECK_ABORT (p4est_quadrant_is_next_D (&F, &E), "is_next_D"); */
+  SC_CHECK_ABORT (!p4est_quadrant_is_next_D (&A, &H), "is_next_D");
 
   p4est_quadrant_parent (&H, &a);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a) == 1, "parent");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a), "parent");
 
   p4est_quadrant_sibling (&P, &h, 7);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&H, &h) == 1, "sibling");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&H, &h), "sibling");
 
   p8est_quadrant_children (&A, &c0, &c1, &c2, &c3, &c4, &c5, &c6, &c7);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c6, &P) == 1, "children");
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c7, &H) == 1, "children");
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c7, &Q) == 0, "children");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c6, &P), "children");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c7, &H), "children");
+  SC_CHECK_ABORT (!p4est_quadrant_is_equal (&c7, &Q), "children");
 
   SC_CHECK_ABORT (p8est_quadrant_is_family (&c0, &c1, &c2, &c3,
-                                            &c4, &c5, &c6, &c7) == 1,
-                  "is_family");
+                                            &c4, &c5, &c6, &c7), "is_family");
   id0 = p4est_quadrant_child_id (&c0);
   id1 = p4est_quadrant_child_id (&c1);
   id2 = p4est_quadrant_child_id (&c2);
@@ -654,28 +661,35 @@ main (int argc, char **argv)
   SC_CHECK_ABORT (p4est_quadrant_child_id (&G) == 7, "child_id");
 
   p4est_quadrant_first_descendant (&A, &c1, 1);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c0, &c1) == 1,
-                  "first_descendant");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&c0, &c1), "first_descendant");
 
   p4est_quadrant_last_descendant (&A, &g, P4EST_QMAXLEVEL - 1);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&Q, &g) == 1, "last_descendant");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&Q, &g), "last_descendant");
 
-  Fid = p4est_quadrant_linear_id (&F, P4EST_QMAXLEVEL);
-  p4est_quadrant_set_morton (&f, P4EST_QMAXLEVEL, Fid);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&F, &f) == 1,
-                  "set_morton/linear_id");
+  p4est_quadrant_linear_id_ext128 (&F, P4EST_QMAXLEVEL, &Fid);
+  p4est_quadrant_set_morton_ext128 (&f, P4EST_QMAXLEVEL, &Fid);
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&F, &f), "set_morton/linear_id");
 
   Aid = p4est_quadrant_linear_id (&A, 0);
   p4est_quadrant_set_morton (&a, 0, Aid);
   SC_CHECK_ABORT (Aid == 27, "linear_id");
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a) == 1,
-                  "set_morton/linear_id");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a), "set_morton/linear_id");
+
+  check_successor_predecessor (&F);
+  check_predecessor_successor (&F);
+  check_predecessor_successor (&G);
+  check_predecessor_successor (&H);
+  check_successor_predecessor (&I);
+  check_predecessor_successor (&I);
+  check_successor_predecessor (&P);
+  check_predecessor_successor (&P);
+  check_predecessor_successor (&Q);
 
   p4est_nearest_common_ancestor (&P, &H, &a);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a) == 1, "ancestor");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a), "ancestor");
 
   p4est_nearest_common_ancestor_D (&P, &Q, &a);
-  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a) == 1, "ancestor_D");
+  SC_CHECK_ABORT (p4est_quadrant_is_equal (&A, &a), "ancestor_D");
 
   for (k = 0; k < 27; ++k) {
     p4est_quadrant_set_morton (&E, 0, (uint64_t) indices[k]);
